@@ -31,7 +31,11 @@
 #include <kencodingfiledialog.h>
 #include <krun.h>
 #include <kstandarddirs.h>
+#include <kio/netaccess.h>
+#include <kpushbutton.h>
+#include <kurl.h>
 
+#include "kileuntitled.h"
 #include "templates.h"
 #include "newfilewizard.h"
 #include "managetemplatesdialog.h"
@@ -49,7 +53,6 @@
 #include "kilestdtools.h"
 #include "kilelistselector.h"
 #include "kiletoolmanager.h"
-#include "kileautosavejob.h"
 #include "kilekonsolewidget.h"
 #include "kileconfig.h"
 #include "kilelogwidget.h"
@@ -68,7 +71,7 @@ Manager::Manager(KileInfo *info, QObject *parent, const char *name) :
 
 	if ( KileConfig::defaultEncoding() == "invalid" )
 		KileConfig::setDefaultEncoding(QString::fromLatin1(QTextCodec::codecForLocale()->name()));
-	
+
 	QWidget *par = m_ki ? m_ki->parentWidget() : 0;
 	m_kpd = new KProgressDialog(par, 0, i18n("Open Project..."), QString::null, true);
 	m_kpd->showCancelButton(false);
@@ -86,17 +89,17 @@ Manager::~Manager()
 void Manager::trashDoc(Info *docinfo, Kate::Document *doc /*= 0L*/ )
 {
 	kdDebug() << "==void Manager::trashDoc(" << docinfo->url().path() << ")=====" << endl;
-	
+
 	if ( m_ki->isOpen(docinfo->url()) ) return;
 
 	if ( doc == 0L ) doc = docinfo->getDoc();
 	//look for doc before we detach the docinfo
 	//if we do it the other way around, docFor will always return nil
  	if ( doc == 0L ) doc = docFor(docinfo->url());
-	
+
 	kdDebug() << "DETACHING " << docinfo << endl;
 	docinfo->detach();
-	
+
 	kdDebug() << "\tTRASHING " <<  doc  << endl;
 	if ( doc == 0L ) return;
 
@@ -116,12 +119,23 @@ void Manager::trashDoc(Info *docinfo, Kate::Document *doc /*= 0L*/ )
 	delete doc;
 }
 
+// update all Info's with changed user commands
+void Manager::updateInfos()
+{
+	for (uint i=0; i < m_infoList.count(); ++i)
+	{
+		m_infoList.at(i)->updateStructLevelInfo();
+	}
+}
+
 Kate::Document* Manager::docFor(const KURL & url)
 {
 	for (uint i=0; i < m_infoList.count(); ++i)
 	{
-		if (m_ki->similarOrEqualURL(m_infoList.at(i)->url(),url))
+		if ( m_ki->similarOrEqualURL(m_infoList.at(i)->url(),url) )
+        {
 			return m_infoList.at(i)->getDoc();
+        }
 	}
 
 	return 0L;
@@ -154,23 +168,20 @@ Info *Manager::infoFor(const QString & path) const
 	return 0L;
 }
 
-Info* Manager::infoFor(Kate::Document* doc, bool usepath /*= true*/ ) const
+Info* Manager::infoFor(Kate::Document* doc) const
 {
-	if (usepath)
-		return infoFor(doc->url().path());
-	else
-	{
-		QPtrListIterator<Info> it(m_infoList);
-		while ( true )
-		{
-			if ( it.current()->getDoc() == doc)
-				return it.current();
-	
-			if (it.atLast()) return 0L;
-	
-			++it;
-		}
-	}
+    QPtrListIterator<Info> it(m_infoList);
+    while ( true )
+    {
+        if ( it.current()->getDoc() == doc)
+            return it.current();
+
+        if (it.atLast()) return 0L;
+
+        ++it;
+    }
+
+    return 0;
 }
 
 void Manager::mapItem(Info *docinfo, KileProjectItem *item)
@@ -268,6 +279,11 @@ KileProjectItemList* Manager::itemsFor(Info *docinfo) const
 	return list;
 }
 
+bool Manager::isProjectOpen()
+{
+	return ( m_projects.count() > 0 );
+}
+
 KileProject* Manager::activeProject()
 {
 	KileProject *curpr=0;
@@ -321,20 +337,20 @@ Info* Manager::createDocumentInfo(const KURL & url)
 
 	if ( docinfo == 0L )
 	{
-		if ( Info::isTeXFile(url) || url.isEmpty() )
+		if ( Info::isTeXFile(url) || url.isEmpty() || KileUntitled::isUntitled(url.url()) )
 		{
 			kdDebug() << "CREATING TeXInfo for " << url.url() << endl;
-			docinfo = new TeXInfo(0L);
+			docinfo = new TeXInfo(0L,m_ki->latexCommands());
 		}
 		else if ( Info::isBibFile(url) )
 		{
 			kdDebug() << "CREATING BibInfo for " << url.url() << endl;
-			docinfo = new BibInfo(0L);
+			docinfo = new BibInfo(0L,m_ki->latexCommands());
 		}
 		else
 		{
 			kdDebug() << "CREATING Info for " << url.url() << endl;
-			docinfo = new Info(0L);
+			docinfo = new Info(0L,m_ki->latexCommands());
 		}
 		docinfo->setURL(url);
 		emit(documentInfoCreated(docinfo));
@@ -349,20 +365,21 @@ Info* Manager::recreateDocInfo(Info *oldinfo, const KURL & url)
 {
 	KileProjectItemList *list = itemsFor(oldinfo);
 	Info *newinfo = createDocumentInfo(url);
- 
+
     newinfo->setDoc(oldinfo->getDoc());
 
 	KileProjectItem *pritem = 0L;
 	for ( pritem = list->first(); pritem; pritem = list->next() )
 		pritem->setInfo(newinfo);
-	
-	m_infoList.removeRef(oldinfo);
+
+	m_infoList.remove(oldinfo);
 	delete oldinfo;
 
-	m_infoList.append(newinfo);
+	//m_infoList.append(newinfo); done in createDocumentInfo
 
     // parse the document
     newinfo->updateStruct();
+	emit(updateStructure(false, newinfo));
 
 	return newinfo;
 }
@@ -371,14 +388,22 @@ bool Manager::removeDocumentInfo(Info *docinfo, bool closingproject /* = false *
 {
 	kdDebug() << "==Manager::removeDocumentInfo(Info *docinfo)=====" << endl;
 	KileProjectItemList *itms = itemsFor(docinfo);
+	bool oneItem = false;
+	if (itms->count() == 1)
+		oneItem = true;
 
-	if ( itms->count() == 0 || (closingproject && itms->count() == 1))
+	if ( itms->count() == 0 || ( closingproject && oneItem ))
 	{
 		kdDebug() << "\tremoving " << docinfo <<  " count = " << m_infoList.count() << endl;
 		m_infoList.remove(docinfo);
 
 		emit ( closingDocument ( docinfo ) );
 
+		if(oneItem)
+		{		
+			itms->first()->setInfo(0L); // really ugly hack, but otherwise item->getInfo() still gives an docinfo != 0L even after delete docinfo (tbraun)
+		}
+		
 		delete docinfo;
 		delete itms;
 
@@ -404,18 +429,20 @@ Kate::Document* Manager::createDocument(Info *docinfo, const QString & encoding,
 	QString enc = encoding.isNull() ? KileConfig::defaultEncoding() : encoding;
 	doc->setEncoding(enc);
 
-	kdDebug() << "opening url: " << docinfo->url().path() << endl;
+    kdDebug() << "url is = " << docinfo->url().url() << endl;
 
-	doc->openURL(docinfo->url());
-	//TODO: connect to completed() signal, now updatestructure is called before loading is completed
+    doc->openURL(docinfo->url());
 
-	if ( !docinfo->url().isEmpty() )
-	{
-		doc->setDocName(docinfo->url().path());
-		emit(addToRecentFiles(docinfo->url()));
-	}
-	else
-		doc->setDocName(i18n("Untitled"));
+    if ( KileUntitled::isUntitled(docinfo->url().url()) )
+    {
+        doc->setDocName(docinfo->url().url());
+        kdDebug() << "docName = " << doc->docName() << endl;
+    }
+    else
+    {
+        doc->setDocName(docinfo->url().path());
+        emit(addToRecentFiles(docinfo->url()));
+    }
 
 	setHighlightMode(doc, highlight);
 
@@ -458,7 +485,7 @@ void Manager::setHighlightMode(Kate::Document * doc, const QString &highlight)
 		if (found) doc->setHlMode(mode);
 		else doc->setHlMode(nHlLaTeX);
 	}
-	else if ( doc->url().isEmpty() || doc->docName() == i18n("Untitled") )
+	else if ( doc->url().isEmpty() || KileUntitled::isUntitled(doc->docName()) )
 	{
 		doc->setHlMode(nHlLaTeX);
 	}
@@ -499,9 +526,9 @@ Kate::View* Manager::loadItem(KileProjectItem *item, const QString & text)
 
 Kate::View* Manager::load(const KURL &url , const QString & encoding /* = QString::null */, bool create /* = true */, const QString & highlight /* = QString::null */, const QString & text /* = QString::null */)
 {
-	kdDebug() << "==load()=================" << endl;
+	kdDebug() << "==load(" << url.url() << ")=================" << endl;
 	//if doc already opened, update the structure view and return the view
-	if ( url.path() != i18n("Untitled") && m_ki->isOpen(url))
+	if ( m_ki->isOpen(url))
 		return m_ki->viewManager()->switchToView(url);
 
 	Info *docinfo = createDocumentInfo(url);
@@ -550,7 +577,7 @@ Kate::View* Manager::loadTemplate(TemplateItem *sel)
 
 Kate::View* Manager::createDocumentWithText(const QString & text)
 {
-	Kate::View *view = load(KURL(), QString::null, true, QString::null, text);
+	Kate::View *view = load(KURL::fromPathOrURL(KileUntitled::next()), QString::null, true, QString::null, text);
 	if (view)
 	{
 		//FIXME this shouldn't be necessary!!!
@@ -604,6 +631,7 @@ void Manager::fileNew()
 		loadTemplate(nfw->getSelection());
 		if ( nfw->useWizard() ) emit ( startWizard() );
 		emit(updateStructure(false, 0L));
+		emit(updateModeStatus());
 	}
 	delete nfw;
 }
@@ -627,7 +655,7 @@ void Manager::fileOpen()
         currentDir = QFileInfo(compileName).dirPath(true);
     else
         currentDir = m_ki->fileSelector()->dirOperator()->url().path();
-    
+
 	QString filter;
 	filter.append(SOURCE_EXTENSIONS);
 	filter.append(" ");
@@ -673,7 +701,8 @@ void Manager::slotNameChanged(Kate::Document * doc)
 	kdDebug() << "==Kile::slotNameChanged==========================" << endl;
 
 	KURL validURL = Info::makeValidTeXURL(doc->url());
-	if(validURL != doc->url()) 
+	bool changedURL = (validURL != doc->url());
+	if(changedURL)
 	{
 		QFile oldFile(doc->url().path());
 		if(doc->saveAs(validURL))
@@ -684,12 +713,12 @@ void Manager::slotNameChanged(Kate::Document * doc)
 
 	Info *docinfo = infoFor(doc);
 
-	// take special care if doc was Untitled before
-	if (docinfo->oldURL().isEmpty())
+	// take special care if doc was Untitled before, or an extension was added by makeValidTeXURL
+	if (docinfo->oldURL().isEmpty() || changedURL)
 	{
 		kdDebug() << "\tadding URL to projectview " << doc->url().path() << endl;
 		m_ki->viewManager()->projectView()->add(doc->url());
-		
+
         recreateDocInfo(docinfo, doc->url());
 	}
 
@@ -710,40 +739,47 @@ void Manager::newDocumentStatus(Kate::Document *doc)
 		emit(updateStructure(true, infoFor(doc)));
 }
 
-void Manager::fileSaveAll(bool amAutoSaving)
+void Manager::fileSaveAll(bool amAutoSaving, bool disUntitled )
 {
 	Kate::View *view;
 	QFileInfo fi;
-
-	kdDebug() << "==Kile::fileSaveAll=================" << endl;
-	kdDebug() << "\tautosaving = " << amAutoSaving << endl;
+	bool successBackup;
+	int saveResult;
+	QString backupFileName;
+	KURL url;
+	
+	kdDebug() << "===Kile::fileSaveAll=================" << endl;
+	kdDebug() << "autosaving = " << amAutoSaving << ", DisUntitled = " << disUntitled << endl;
 
 	for (uint i = 0; i < m_ki->viewManager()->views().count(); ++i)
 	{
 		view = m_ki->viewManager()->view(i);
 
-		if (view && (view->getDoc()->isModified() || amAutoSaving) )
+		if ( view && view->getDoc()->isModified() )
 		{
-			fi.setFile(view->getDoc()->url().path());
-			//don't save unwritable and untitled documents when autosaving
-			if (
-			      (!amAutoSaving) ||
-				  (amAutoSaving && (!view->getDoc()->url().isEmpty() ) && fi.isWritable() )
-			   )
+			url = view->getDoc()->url();
+			fi.setFile(url.path());
+			backupFileName = url.path()+ ".backup";
+			
+			if	( 	( !amAutoSaving && !(disUntitled && url.isEmpty() ) ) // DisregardUntitled is true and we have an untitled doc and don't autosave
+					|| ( amAutoSaving && !url.isEmpty() ) //don't save untitled documents when autosaving
+					|| ( !amAutoSaving && !disUntitled )	// both false, so we want to save everything
+				)
 			{
-				kdDebug() << "\tsaving: " << view->getDoc()->url().path() << endl;
-
-				if (amAutoSaving)
+				if (amAutoSaving && fi.size() > 0) // the size check ensures that we don't save empty files (to prevent something like #125809 in the future).
 				{
-					//make a backup
-					KURL url = view->getDoc()->url();
-					KileAutoSaveJob *job = new KileAutoSaveJob(url);
-
-					//save the current file if job is finished succesfully
-					if (view->getDoc()->isModified()) connect(job, SIGNAL(success()), view, SLOT(save()));
+					kdDebug() << "autosaving: " << backupFileName << endl;
+					successBackup = KIO::NetAccess::file_copy(url, KURL::fromPathOrURL(backupFileName), -1, true, false, kapp->mainWidget());
+					if(!successBackup)
+						m_ki->logWidget()->printMsg(KileTool::Error,i18n("The file %1 could not be saved, check the permissions and the free disk space!").arg(backupFileName),i18n("Autosave"));
 				}
-				else
-					view->save();
+				
+				kdDebug() << "saving: " << url.path() << endl;
+				saveResult = view->save();
+				fi.refresh();
+			
+				if(saveResult == Kate::View::SAVE_ERROR && fi.size() == 0  && !url.isEmpty()) // we probably hit bug #125809, inform the user of the possible consequences
+					m_ki->logWidget()->printMsg(KileTool::Error,i18n("Kile encountered problems while saving the file %1. Do you have enough free disk space left?").arg(url.url()),i18n("Saving"));
 			}
 		}
 	}
@@ -751,25 +787,47 @@ void Manager::fileSaveAll(bool amAutoSaving)
 
 void Manager::fileOpen(const KURL & url, const QString & encoding)
 {
-	//don't want to receive signals from the fileselector since 
+	//don't want to receive signals from the fileselector since
 	//that would allow the user to open a single file twice by double-clicking on it
-	m_ki->fileSelector()->blockSignals(true); 
-	
+	m_ki->fileSelector()->blockSignals(true);
+
 	kdDebug() << "==Kile::fileOpen==========================" << endl;
 	kdDebug() << "\t" << url.url() << endl;
 	bool isopen = m_ki->isOpen(url);
 
-	load(url, encoding);
+	Kate::View *view = load(url, encoding);
+	KileProjectItem *item = itemFor(url);
 
-	//URL wasn't open before loading, add it to the project view
-	//FIXME: use signal/slot
-	if (!isopen && (itemFor(url) == 0) ) m_ki->viewManager()->projectView()->add(url);
+	if(!isopen)
+	{
+		if(!item)	//URL wasn't open before loading, add it to the project view
+			m_ki->viewManager()->projectView()->add(url);	//FIXME: use signal/slot
+		else if(view)
+			view->setCursorPosition(item->lineNumber(),item->columnNumber());
+	}
 
 	emit(updateStructure(false, 0L));
 	emit(updateModeStatus());
+	// update undefined references in this file
+	emit(updateReferences(infoFor(url.path())) );
 	m_ki->fileSelector()->blockSignals(false);
 }
 
+bool Manager::fileCloseAllOthers()
+{
+	Kate::View * currentview = m_ki->viewManager()->currentView();
+	while (  m_ki->viewManager()->views().count() > 1 )
+	{
+		Kate::View *view =  m_ki->viewManager()->views().first();
+		if ( view == currentview )
+			view =  m_ki->viewManager()->views().next();
+		if ( ! fileClose(view->getDoc()) )
+			return false;
+	}
+
+	return true;
+}
+
 bool Manager::fileCloseAll()
 {
 	Kate::View * view = m_ki->viewManager()->currentView();
@@ -829,8 +887,12 @@ bool Manager::fileClose(Kate::Document *doc /* = 0L*/, bool closingproject /*= f
 
 		if ( doc->closeURL() )
 		{
+			// docinfo may have been recreated from 'Untitled' doc to a named doc
+			if ( url .isEmpty() )
+				docinfo= infoFor(doc);
+				
 			if ( KileConfig::cleanUpAfterClose() ) cleanUpTempFiles(docinfo, true);
-			
+
 			//FIXME: use signal/slot
 			m_ki->viewManager()->removeView(static_cast<Kate::View*>(doc->views().first()));
 			//remove the decorations
@@ -842,7 +904,7 @@ bool Manager::fileClose(Kate::Document *doc /* = 0L*/, bool closingproject /*= f
 			//FIXME:remove entry in projectview
 			m_ki->viewManager()->removeFromProjectView(url);
 
-            if ( m_ki->docManager()->documentInfos()->count() == 0 ) emit updateModeStatus();
+            emit updateModeStatus();
 		}
 		else
 			return false;
@@ -898,11 +960,6 @@ void Manager::projectNew()
 		if (dlg->createNewFile())
 		{
 			QString filename = dlg->file();
-			KURL fileURL; fileURL.setFileName(filename);
-			if(KileDocument::Info::containsInvalidCharacters(fileURL)) {
-				KURL newURL = KileDocument::Info::repairInvalidCharacters(fileURL);
-				filename = newURL.fileName();
-			}
 
 			//create the new document and fill it with the template
 			//TODO: shell expand the filename
@@ -931,6 +988,8 @@ void Manager::projectNew()
 		project->buildProjectTree();
 		//project->save();
 		addProject(project);
+		emit(updateModeStatus());
+		emit(addToRecentProjects(project->url()));
 	}
 }
 
@@ -1014,6 +1073,8 @@ void Manager::removeFromProject(const KileProjectItem *item)
 		KileProject *project = item->project();
 		item->project()->remove(item);
 
+		// update undefined references in all project files
+		updateProjectReferences(project);
 		project->buildProjectTree();
 	}
 }
@@ -1064,6 +1125,13 @@ KileProject* Manager::projectOpen(const KURL & url, int step, int max)
 	m_kpd->show();
 
 	KileProject *kp = new KileProject(url);
+	
+	if(kp->isInvalid())
+	{
+		m_kpd->cancel();
+		delete kp;
+		return 0L;
+	}
 
 	emit(addToRecentProjects(url));
 
@@ -1086,19 +1154,31 @@ KileProject* Manager::projectOpen(const KURL & url, int step, int max)
 
 	emit(updateStructure(false, 0L));
 	emit(updateModeStatus());
+	// update undefined references in all project files
+	updateProjectReferences(kp);
 
 	if (step == (max - 1))
 		m_kpd->cancel();
 
     m_ki->viewManager()->switchToView(kp->lastDocument());
-		
+
 	return kp;
 }
 
+// as all labels are gathered in the project, we can check for unsolved references
+void Manager::updateProjectReferences(KileProject *project)
+{
+	KileProjectItemList *list = project->items();
+	for ( uint i=0; i < list->count(); ++i)
+	{
+		emit(updateReferences(list->at(i)->getInfo()));
+	}
+}
+
 KileProject* Manager::projectOpen()
 {
 	kdDebug() << "==Kile::projectOpen==========================" << endl;
-	KURL url = KFileDialog::getOpenURL( "", i18n("*.kilepr|Kile Project Files\n*|All Files"), m_ki->parentWidget(), i18n("Open Project") );
+	KURL url = KFileDialog::getOpenURL( KileConfig::defaultProjectLocation(), i18n("*.kilepr|Kile Project Files\n*|All Files"), m_ki->parentWidget(), i18n("Open Project") );
 
 	if (!url.isEmpty())
 		return projectOpen(url);
@@ -1124,8 +1204,8 @@ void Manager::projectSave(KileProject *project /* = 0 */)
 		KileProjectItemList *list = project->items();
 		Kate::Document *doc = 0L;
 
-		KileProjectItem *item;
-		Info *docinfo;
+		KileProjectItem *item = 0L;
+		Info *docinfo = 0L;
 		//update the open-state of the items
 		for (uint i=0; i < list->count(); ++i)
 		{
@@ -1134,12 +1214,13 @@ void Manager::projectSave(KileProject *project /* = 0 */)
 			item->setOpenState(m_ki->isOpen(item->url()));
 			docinfo = item->getInfo();
 
-			kdDebug() << "test" << endl;
-			kdDebug() << "\t\tDOCINFO = " << docinfo << ", DOC = " << docinfo->getDoc() << endl;
-			if (docinfo != 0L) doc = docinfo->getDoc();
-			if (doc != 0L) storeProjectItem(item, doc);
+			if (docinfo != 0L)
+				doc = docinfo->getDoc();
+			if (doc != 0L)
+				storeProjectItem(item, doc);
 
 			doc = 0L;
+			docinfo = 0L;
 		}
 
 		project->save();
@@ -1176,13 +1257,26 @@ void Manager::projectAddFiles(KileProject *project)
 			if (fi.exists()) currentDir= fi.dirPath();
 		}
 
-		KURL::List urls = KFileDialog::getOpenURLs( currentDir, i18n("*|All Files"), m_ki->parentWidget(), i18n("Add Files") );
+		KFileDialog *dlg = new KFileDialog(currentDir, i18n("*|All Files"),m_ki->parentWidget(), 0, true );
+		KPushButton* okButton = dlg->okButton();
+		okButton->setText(i18n("Add"));
+		dlg->setCaption(i18n("Add Files"));
+		KFile::Mode mode = static_cast<KFile::Mode>( KFile::Files | KFile::ExistingOnly);
+		dlg->setMode(mode);
+
+		if(dlg->exec())
+		{
+			KURL::List urls = dlg->selectedURLs();
+			for (uint i=0; i < urls.count(); ++i)
+			{
+				addToProject(project, urls[i]);
+			}
+			// update undefined references in all project files
+			updateProjectReferences(project);
+		}
+		delete dlg;
 
 		//open them
-		for (uint i=0; i < urls.count(); ++i)
-		{
-			addToProject(project, urls[i]);
-		}
 	}
 	else if (m_projects.count() == 0)
 		KMessageBox::error(m_ki->parentWidget(), i18n("There are no projects opened. Please open the project you want to add files to, then choose Add Files again."),i18n( "Could Not Determine Active Project"));
@@ -1302,46 +1396,44 @@ bool Manager::projectClose(const KURL & url)
  	if (project)
 	{
 		kdDebug() << "\tclosing:" << project->name() << endl;
-        project->setLastDocument(m_ki->getName());
-
-		//close the project file first, projectSave, changes this file
-		Kate::Document *doc = docFor(project->url());
-		if (doc)
-		{
-			doc->save();
-			fileClose(doc);
-		}
+        	project->setLastDocument(m_ki->getName());
 
 		projectSave(project);
 
 		KileProjectItemList *list = project->items();
 
 		bool close = true;
-		Info *docinfo;
+		Kate::Document *doc = 0L;
+		Info *docinfo = 0L;
 		for (uint i =0; i < list->count(); ++i)
-		{
+		{	
+			doc = 0L;
 			docinfo = list->at(i)->getInfo();
-			if (docinfo) doc = docinfo->getDoc();
+			if (docinfo)
+				doc = docinfo->getDoc();
 			if (doc)
 			{
 				kdDebug() << "\t\tclosing item " << doc->url().path() << endl;
 				bool r = fileClose(doc, true);
 				close = close && r;
-				if (!close) break;
+				if (!close)
+					break;
 			}
-			else
-				removeDocumentInfo(docinfo, true);
-
-			docinfo = 0L;
-			doc = 0L;
 		}
 
 		if (close)
 		{
+			for (uint i =0; i < list->count(); ++i)
+			{
+				docinfo = list->at(i)->getInfo();
+				if (docinfo)
+					removeDocumentInfo(docinfo, true);
+			}
 			m_projects.remove(project);
 			//FIXME: use signal/slot
 			m_ki->viewManager()->projectView()->remove(project);
 			delete project;
+			emit(updateModeStatus());
 			return true;
 		}
 		else
@@ -1383,7 +1475,7 @@ void Manager::cleanUpTempFiles(Info *docinfo, bool silent)
 	}
 
 	str = file.fileName();
-	if (!silent &&  (str==i18n("Untitled") || str.isEmpty() ) )	return;
+	if (!silent &&  ( KileUntitled::isUntitled(str) || str.isEmpty() ) )	return;
 
 	if (!silent && extlist.count() > 0 )
 	{
@@ -1411,6 +1503,205 @@ void Manager::cleanUpTempFiles(Info *docinfo, bool silent)
 	docinfo->cleanTempFiles(extlist);
 }
 
+// Show all opened projects and switch to another one, if you want
+
+void Manager::projectShow()
+{
+	if ( m_projects.count() <= 1 )
+		return;
+
+	// select the new project
+	KileProject *project = selectProject(i18n("Switch Project"));
+	if ( !project || project==activeProject() )
+		return;
+
+	// get last opened document
+	const KURL lastdoc = project->lastDocument();
+	KileProjectItem *docitem = ( !lastdoc.isEmpty() ) ? itemFor(lastdoc,project) : 0L;
+
+	// if not, we search for the first opened tex file of this project
+	// if no file is opened, we take the first tex file mentioned in the list
+	KileProjectItem *first_texitem = 0L;
+	if ( ! docitem ) {
+		KileProjectItemList *list = project->items();
+		for ( KileProjectItem *item=list->first(); item; item=list->next() ) {
+			if ( item->path().find(".tex",-4) >= 0 ) {
+				if  ( m_ki->isOpen(item->url()) ) {
+					docitem = item;
+					break;
+				} else if ( ! first_texitem )
+					first_texitem = item;
+			}
+		}
+	}
+
+	// did we find one opened file or must we take the first entry
+	if ( ! docitem ) {
+		if ( ! first_texitem )
+			return;
+		docitem = first_texitem;
+	}
+
+	// ok, we can switch to another project now
+	if  ( m_ki->isOpen(docitem->url()) )
+		m_ki->viewManager()->switchToView(docitem->url());
+	else
+		fileOpen( docitem->url(),docitem->encoding() );
+}
+
+void Manager::projectRemoveFiles()
+{
+	KileProjectItem *item = selectProjectFileItem( i18n("Select File to Remove") );
+	if ( item ) {
+		removeFromProject(item);
+	}
+}
+
+void Manager::projectShowFiles()
+{
+	KileProjectItem *item = selectProjectFileItem( i18n("Select File") );
+	if ( item )
+	{
+		if  ( item->type() == KileProjectItem::ProjectFile )
+			dontOpenWarning( item, i18n("Show Project Files"), i18n("project configuration file") );
+		else if ( item->type() == KileProjectItem::Image )
+			dontOpenWarning( item, i18n("Show Project Files"), i18n("graphics file") );
+		else
+		{
+			// ok, we can switch to another file
+			if  ( m_ki->isOpen(item->url()) )
+				m_ki->viewManager()->switchToView(item->url());
+			else
+				fileOpen( item->url(),item->encoding() );
+		}
+	}
+}
+
+void Manager::projectOpenAllFiles()
+{
+	KileProject *project = selectProject(i18n("Select Project"));
+	if (project != 0L)
+  {
+		projectOpenAllFiles(project->url());
+  }
+}
+
+void Manager::projectOpenAllFiles(const KURL & url)
+{
+	KileProject* project;
+	Kate::Document* doc = 0L;
+
+	if(!url.isValid())
+		return;
+	project = projectFor(url);
+
+	if(!project)
+		return;
+
+
+	if(m_ki->viewManager()->currentView())
+		doc = m_ki->viewManager()->currentView()->getDoc();
+	// we remember the actual view, so the user gets the same view back after opening
+
+	KileProjectItemList *list = project->items();
+	for ( KileProjectItem *item=list->first(); item; item = list->next() )
+	{
+		if  ( item->type()==KileProjectItem::ProjectFile )
+			dontOpenWarning( item, i18n("Open All Project Files"), i18n("project configuration file") );
+		else if  ( item->type()==KileProjectItem::Image )
+			dontOpenWarning( item, i18n("Open All Project Files"), i18n("graphics file") );
+		else if ( ! m_ki->isOpen(item->url()) )
+			fileOpen( item->url(),item->encoding() );
+	}
+
+	if(doc) // we have a doc so switch back to original view
+		m_ki->viewManager()->switchToView(doc->url());
+}
+
+QStringList Manager::getProjectFiles()
+{
+	QStringList filelist;
+
+	KileProject *project = activeProject();
+	if ( project )
+	{
+		KileProjectItemList *list = project->items();
+		for ( KileProjectItem *item=list->first(); item; item = list->next() )
+		{
+			if  ( item->type()!=KileProjectItem::ProjectFile && item->type()!=KileProjectItem::Image )
+				filelist <<  item->url().path();
+		}
+	}
+	return filelist;
+}
+
+void Manager::dontOpenWarning(KileProjectItem *item, const QString &action, const QString &filetype)
+{
+	m_ki->logWidget()->printMsg(KileTool::Info, i18n("not opened: %1 (%2)").arg(item->url().path()).arg(filetype), action);
+}
+
+KileProjectItem* Manager::selectProjectFileItem(const QString &label)
+{
+	// select a project
+	KileProject *project = selectProject(i18n("Select Project"));
+	if ( ! project )
+		return 0L;
+
+	// get a list of files
+	QStringList filelist;
+	QMap<QString,KileProjectItem *> map;
+	KileProjectItemList *list = project->items();
+	for ( KileProjectItem *item=list->first(); item; item = list->next() ) {
+		filelist << item->path();
+		map[item->path()] = item;
+	}
+
+	// select one of these files
+	KileProjectItem *item = 0L;
+	KileListSelector *dlg  = new KileListSelector(filelist,i18n("Project Files"),label, m_ki->parentWidget());
+	if ( dlg->exec() ) {
+		if ( dlg->currentItem() >= 0 ) {
+			QString name = filelist[dlg->currentItem()];
+			if ( map.contains(name) )
+				item = map[name];
+			else
+				KMessageBox::error(m_ki->parentWidget(), i18n("Could not determine the selected file."),i18n( "Project Error"));
+		}
+	}
+	delete dlg;
+
+	return item;
+}
+
+// add a new file to the project
+//  - only when there is an active project
+//  - if the file doesn't already belong to it (checked by addToProject)
+
+void Manager::projectAddFile(QString filename, bool graphics)
+{
+ 	kdDebug() << "===Kile::projectAddFile==============" << endl;
+	KileProject *project = activeProject();
+ 	if ( ! project )
+		return;
+
+	if ( ! QFileInfo(filename).exists() )
+	{
+		if ( graphics )
+			return;
+		filename += ".tex";
+		if ( ! QFileInfo(filename).exists() )
+			return;
+	}
+
+	//ok, we have a project and an existing file
+	kdDebug() << "\tadd file: " << filename << endl;
+	m_ki->viewManager()->updateStructure(true);
+
+	KURL url;
+	url.setPath(filename);
+	addToProject(project, url);
+}
+
 }
 
 #include "kiledocmanager.moc"
