@@ -1,7 +1,9 @@
 /***************************************************************************
     begin                : Sun Jul 20 2003
     copyright            : (C) 2003 by Jeroen Wijnhout
+                               2005 by Holger Danielsson
     email                : Jeroen.Wijnhout@kdemail.net
+                           holger.danielsson@t-online.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -13,6 +15,33 @@
  *                                                                         *
  ***************************************************************************/
 
+// 2005-11-02: dani
+//  - cleaning up source of central function updateStruct()
+//      - always use 'else if', because all conditions are exclusive or
+//      - most often used commands are at the top
+//  - add some new types of elements (and levels) for the structure view
+//  - new commands, which are passed to the structure listview:
+//       \includegraphics, \caption
+//  - all user defined commands for labels are recognized
+//  - changed folder name of KileStruct::BibItem to "bibs", so that "refs"
+//    is still unused and can be used for references (if wanted)
+//  - \begin, \end to gather all environments. But only figure and table 
+//    environments are passed to the structure view
+
+// 2005-11-26: dani
+//  - add support for \fref, \Fref and \eqref references commands
+
+// 2005-12-07: dani
+//  - add support to enable and disable some structure view items
+
+// 2006-01-16 tbraun
+// - fix #59945 Now we call (through a signal ) project->buildProjectTree so the bib files are correct,
+//   and therefore the keys in \cite completion
+
+// 2006-02-09 tbraun/dani
+// - fix #106261#4 improved parsing of (optional) command parameters
+// - all comments are removed
+
 #include <qfileinfo.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -29,6 +58,8 @@
 
 #include "codecompletion.h"
 #include "kiledocumentinfo.h"
+#include "kileuntitled.h"
+#include "kileconfig.h"
 
 namespace KileDocument
 {
@@ -37,7 +68,7 @@ bool Info::isTeXFile(const KURL & url)
 {
 	//TODO use mimetype
 	QString shortName = url.fileName();
-	return (shortName.right(4) == ".tex" || shortName.right(4) == ".sty" || shortName.right(4) == ".cls" || shortName.right(4) == ".dtx" || shortName.right(4) == ".ltx" || shortName.right(6) == ".latex") && (shortName != i18n("Untitled"));
+	return (shortName.right(4) == ".tex" || shortName.right(4) == ".sty" || shortName.right(4) == ".cls" || shortName.right(4) == ".dtx" || shortName.right(4) == ".ltx" || shortName.right(6) == ".latex") && ( !KileUntitled::isUntitled(shortName) );
 }
 
 bool Info::isBibFile(const KURL & url)
@@ -58,30 +89,35 @@ KURL Info::repairInvalidCharacters(const KURL& url)
 	do {
 		bool isOK;
 		QString newURL = KInputDialog::getText(
-			i18n("Invalid characters"),
+			i18n("Invalid Characters"),
 			i18n("The filename contains invalid characters ($~ #).<br>Please provide \
-				another one, or click on \"Cancel\" to save anyway."),
-			url.filename(),
+				another one, or click \"Cancel\" to save anyway."),
+			ret.filename(),
 			&isOK);
 		if(!isOK)
-			return ret;
+			break;
 		ret.setFileName(newURL);
 	} while(containsInvalidCharacters(ret));
 	
+	return renameIfExist(ret);
+}
+
+KURL Info::renameIfExist(const KURL& url)
+{	
+	KURL ret(url);
 	while ( QFileInfo(ret.path()).exists() )
 	{
 		bool isOK;
 		QString newURL = KInputDialog::getText(
-			i18n("File already exists."),
-			i18n("A file with filename %1 already exists.<br>Please provide \
-				another one, or click on \"Cancel\" to overwrite.").arg(ret.fileName()),
-			url.filename(),
+			i18n("File Already Exists"),
+			i18n("A file with filename '%1' already exists.<br>Please provide \
+				another one, or click \"Cancel\" to overwrite it.").arg(ret.fileName()),
+			ret.filename(),
 			&isOK);
 		if(!isOK)
-			return ret;
+			break;
 		ret.setFileName(newURL);
 	}
-
 	return ret;
 }
 
@@ -95,14 +131,14 @@ KURL Info::repairExtension(const KURL& url)
 
 	if(KMessageBox::Yes == KMessageBox::questionYesNo(NULL,
 		i18n("The given filename has no extension; do you want one to be automatically added?"),
-		i18n("Missing extension"),
+		i18n("Missing Extension"),
 		KStdGuiItem::yes(),
 		KStdGuiItem::no(),
 		"AutomaticallyAddExtension"))
 	{
 		ret.setFileName(filename + ".tex");	
 	}
-	return ret;
+	return renameIfExist(ret);
 }
 
 KURL Info::makeValidTeXURL(const KURL & url)
@@ -120,23 +156,49 @@ KURL Info::makeValidTeXURL(const KURL & url)
 	return newURL;
 }
 
-Info::Info(Kate::Document *doc) : m_doc(doc)
+Info::Info(Kate::Document *doc, LatexCommands *commands) : m_doc(doc),m_commands(commands)
 {
 	m_config = kapp->config();
 	if (m_doc)
 		kdDebug() << "Info created for " << m_doc->docName() << endl;
 
 	m_bIsRoot = false;
-	m_arStatistics = new long[5];
+	m_arStatistics = new long[SIZE_STAT_ARRAY];
 
 	if (m_doc)
 		m_url=m_oldurl = doc->url();
 	else
 		m_url = m_oldurl = KURL();
+		
+	// initialize m_dictStructLevel
+	updateStructLevelInfo();
+}
 
+Info::~Info(void)
+{
+	kdDebug() << "DELETING DOCINFO" << m_url.path() << endl;
+	delete [] m_arStatistics;
+}
+
+// set struct level dictionary with standard and user defined commands
+void Info::updateStructLevelInfo()
+{	
+	// read config for structview items
+	m_showStructureLabels = KileConfig::svShowLabels();
+	m_showStructureReferences = KileConfig::svShowReferences();
+	m_showStructureBibitems = KileConfig::svShowBibitems();
+	m_showStructureGraphics = KileConfig::svShowGraphics();
+	m_showStructureFloats = KileConfig::svShowFloats();
+	m_openStructureLabels = KileConfig::svOpenLabels();
+	m_openStructureReferences = KileConfig::svOpenReferences();
+	m_openStructureBibitems = KileConfig::svOpenBibitems();
+
+	// clear all entries and rebuild them
+	m_dictStructLevel.clear();
+
+	// add standard commands
 	//TODO: make this configurable
-	m_dictStructLevel["\\label"]= KileStructData(KileStruct::NotSpecified, KileStruct::Label, QString::null, "labels");
-	m_dictStructLevel["\\bibitem"]= KileStructData(KileStruct::NotSpecified, KileStruct::BibItem, QString::null, "refs");
+	// sectioning
 	m_dictStructLevel["\\input"]=KileStructData(KileStruct::File, KileStruct::Input, "include");
 	m_dictStructLevel["\\Input"]=KileStructData(KileStruct::File, KileStruct::Input, "include");
 	m_dictStructLevel["\\include"]=KileStructData(0, KileStruct::Input, "include");
@@ -148,14 +210,70 @@ Info::Info(Kate::Document *doc) : m_doc(doc)
 	m_dictStructLevel["\\paragraph"]=KileStructData(6, KileStruct::Sect, "subsubsection");
 	m_dictStructLevel["\\subparagraph"]=KileStructData(7, KileStruct::Sect, "subsubsection");
 	m_dictStructLevel["\\bibliography"]=KileStructData(0,KileStruct::Bibliography, "bibtex");
+	
+	// hidden commands  
 	m_dictStructLevel["\\usepackage"]=KileStructData(KileStruct::Hidden, KileStruct::Package);
 	m_dictStructLevel["\\newcommand"]=KileStructData(KileStruct::Hidden, KileStruct::NewCommand);
-}
+	m_dictStructLevel["\\addunit"]=KileStructData(KileStruct::Hidden, KileStruct::NewCommand); // hack to get support for the fancyunits package until we can configure the commands in the gui (tbraun)
+	m_dictStructLevel["\\DeclareMathOperator"]=KileStructData(KileStruct::Hidden, KileStruct::NewCommand); // amsmath package
+	m_dictStructLevel["\\caption"]=KileStructData(KileStruct::Hidden,KileStruct::Caption);
 
-Info::~Info(void)
-{
-	kdDebug() << "DELETING DOCINFO" << m_url.path() << endl;
-	delete [] m_arStatistics;
+	// labels, we always gather them here to get codecompl and undefined references
+	m_dictStructLevel["\\label"]= KileStructData(KileStruct::NotSpecified, KileStruct::Label, QString::null, "labels");
+
+	// add user defined commands for labels
+	QStringList labellist;
+	QStringList::ConstIterator it;
+	m_commands->commandList(labellist,KileDocument::CmdAttrLabel,true);
+	for ( it=labellist.begin(); it != labellist.end(); ++it ) 
+	{
+		m_dictStructLevel[*it]= KileStructData(KileStruct::NotSpecified, KileStruct::Label, QString::null, "labels");
+	}
+
+	// bibitems
+	if ( m_showStructureBibitems )
+	{
+		m_dictStructLevel["\\bibitem"]= KileStructData(KileStruct::NotSpecified, KileStruct::BibItem, QString::null, "bibs");
+	}
+	
+	// graphics
+	if ( m_showStructureGraphics )
+	{
+		m_dictStructLevel["\\includegraphics"]=KileStructData(KileStruct::Object,KileStruct::Graphics, "graphics");
+	}
+	
+	// references
+	if ( m_showStructureReferences )
+	{
+		m_dictStructLevel["\\ref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\pageref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\vref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\vpageref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\fref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\Fref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+		m_dictStructLevel["\\eqref"]=KileStructData(KileStruct::Hidden,KileStruct::Reference);
+	
+		// add user defined commands for references
+		QStringList reflist;
+		QStringList::ConstIterator it;
+		m_commands->commandList(reflist,KileDocument::CmdAttrReference,true);
+		for ( it=reflist.begin(); it != reflist.end(); ++it ) 
+		{
+			m_dictStructLevel[*it]= KileStructData(KileStruct::Hidden, KileStruct::Reference);
+		}
+	}
+
+	// float environments
+	if ( m_showStructureFloats )
+	{
+		m_dictStructLevel["\\begin"]=KileStructData(KileStruct::Object,KileStruct::BeginEnv);
+		m_dictStructLevel["\\end"]=KileStructData(KileStruct::Hidden,KileStruct::EndEnv);
+	
+		// some entries, which could never be found (but they are set manually)
+		m_dictStructLevel["\\begin{figure}"]=KileStructData(KileStruct::Object,KileStruct::BeginFloat, "frame_image");
+		m_dictStructLevel["\\begin{table}"]=KileStructData(KileStruct::Object,KileStruct::BeginFloat, "frame_spreadsheet");
+		m_dictStructLevel["\\end{float}"]=KileStructData(KileStruct::Hidden,KileStruct::EndFloat);
+	}
 }
 
 void Info::emitNameChanged(Kate::Document * /*doc*/)
@@ -167,7 +285,7 @@ void Info::emitNameChanged(Kate::Document * /*doc*/)
 		kdDebug() << "\tto: " << m_doc->url().path() << endl;
 
 		//don't emit if new URL is empty (i.e. when closing the document)
-		if (!m_doc->url().isEmpty() && (m_url != m_doc->url() ) )
+		if (!m_doc->url().isEmpty() && QFile::exists(m_doc->url().path()) && (m_url != m_doc->url() ) )
 		{
 			kdDebug() << "\temitting nameChanged(url)" << endl;
 			setURL(m_doc->url());
@@ -181,7 +299,7 @@ void Info::count(const QString line, long *stat)
 {
 	QChar c;
 	int state = stStandard;
-	bool word = false;
+	bool word = false; // we are in a word
 
 	for (uint p=0; p < line.length(); ++p)
 	{
@@ -194,8 +312,7 @@ void Info::count(const QString line, long *stat)
 			{
 				case TEX_CAT0	:
 					state = stControlSequence;
-					++stat[1];
-					++stat[4];
+					++stat[1];		
 
 					//look ahead to avoid counting words like K\"ahler as two words
 					if (! line[p+1].isPunct() || line[p+1] == '~' || line[p+1] == '^' )
@@ -203,8 +320,7 @@ void Info::count(const QString line, long *stat)
 				break;
 
 				case TEX_CAT14 :
-					p=line.length();
-					word=false;
+					state=stComment;
 				break;
 
 				default:
@@ -216,7 +332,6 @@ void Info::count(const QString line, long *stat)
 							word=true;
 							++stat[3];
 						}
-
 						++stat[0];
 					}
 					else
@@ -229,19 +344,48 @@ void Info::count(const QString line, long *stat)
 			}
 		break;
 
-		case stControlSequence	:
-			if ( c.isLetter() )	state = stCommand;
-			else state = stStandard;
-			++stat[1];
+		case stControlSequence :
+			if ( c.isLetter() )
+			{
+			// "\begin{[a-zA-z]+}" is an environment, and you can't define a command like \begin
+				if ( line.mid(p,5) == "begin" )
+				{
+					++stat[5];
+					state = stEnvironment;
+					stat[1] +=5;
+					p+=4; // after break p++ is executed
+				}
+				else if ( line.mid(p,3) == "end" )
+				{
+					stat[1] +=3;
+					state = stEnvironment;
+					p+=2;	
+				} // we don't count \end as new environment, this can give wrong results in selections
+				else
+				{
+					++stat[4];
+					++stat[1];
+					state = stCommand;
+				}
+			}
+			else
+			{
+				++stat[4];
+				++stat[1];
+				state = stStandard;
+			}
 		break;
 
 		case stCommand :
-			if ( c.isLetter() ) { ++stat[1]; }
-			else if ( c == TEX_CAT0 ) ++stat[4];
-			else if ( c == TEX_CAT14 )
+			if ( c.isLetter() )
+				++stat[1];
+			else if ( c == TEX_CAT0 )
 			{
-				p=line.length();
+				++stat[1];
+				state=stControlSequence;
 			}
+			else if ( c == TEX_CAT14 )
+				state=stComment;
 			else
 			{
 				++stat[2];
@@ -249,6 +393,28 @@ void Info::count(const QString line, long *stat)
 			}
 		break;
 
+		case stEnvironment :
+			if ( c == TEX_CAT2  ) // until we find a closing } we have an environment
+			{
+				++stat[1];				
+				state=stStandard;
+			}
+			else if ( c == TEX_CAT14 )
+				state=stComment;
+			else
+				++stat[1];
+				
+		break;
+		
+		case stComment : // if we get a selection the line possibly contains \n and so the comment is only valid till \n and not necessarily till line.length()
+			if ( c == '\n')
+			{
+			++stat[2]; // \n was counted as punctuation in the old implementation
+			state=stStandard;
+			word=false;
+			}
+		break;
+
 		default :
 			kdWarning() << "Unhandled state in getStatistics " << state << endl;
 		break;
@@ -258,9 +424,10 @@ void Info::count(const QString line, long *stat)
 
 const long* Info::getStatistics()
 {
-	//#c in words    , #c in commands,  #c whitespace,    #words,           #latex_commands
-	m_arStatistics[0]=m_arStatistics[1]=m_arStatistics[2]=m_arStatistics[3]=m_arStatistics[4]=0;
-
+	/* [0] = #c in words, [1] = #c in latex commands and environments,
+	   [2] = #c whitespace, [3] = #words, [4] = # latex_commands, [5] = latex_environments */
+	m_arStatistics[0]=m_arStatistics[1]=m_arStatistics[2]=m_arStatistics[3]=m_arStatistics[4]=m_arStatistics[5]=0;
+	
 	return m_arStatistics;
 }
 
@@ -327,7 +494,7 @@ QString Info::matchBracket(QChar obracket, uint &l, uint &pos)
 
 	while ( l <= m_doc->numLines() )
 	{
-		line = m_doc->textLine(l);
+		line = getTextline(l);
 		len = line.length();
 		for (int i=pos; i < len; ++i)
 		{
@@ -337,7 +504,10 @@ QString Info::matchBracket(QChar obracket, uint &l, uint &pos)
 			{
 				--count;
 				if (count < 0)
+				{
+					pos = i;
 					return grab;
+				}
 			}
 
 			grab += line[i];
@@ -366,16 +536,44 @@ void Info::updateBibItems()
 {
 }
 
+QString Info::getTextline(uint line)
+{
+	static QRegExp::QRegExp reComments("[^\\\\](%.*$)");
+
+	QString s = m_doc->textLine(line);
+	if ( ! s.isEmpty() )
+	{
+		// remove comment lines
+		if ( s[0] == '%' )
+		{
+			s = QString::null;
+		}
+		else
+		{
+			//remove escaped \ characters
+			s.replace("\\\\", "  ");
+
+			//remove comments
+			if( s.find(reComments) != -1 )
+			{
+				s = s.left(reComments.pos(1));
+			}
+		}
+	}
+	return s;
+}
+
 const long* TeXInfo::getStatistics()
 {
-	//#c in words    , #c in commands,  #c whitespace,    #words,           #latex_commands
-	m_arStatistics[0]=m_arStatistics[1]=m_arStatistics[2]=m_arStatistics[3]=m_arStatistics[4]=0;
+	/* [0] = #c in words, [1] = #c in latex commands and environments,
+	   [2] = #c whitespace, [3] = #words, [4] = # latex_commands, [5] = latex_environments */
+	m_arStatistics[0]=m_arStatistics[1]=m_arStatistics[2]=m_arStatistics[3]=m_arStatistics[4]=m_arStatistics[5]=0;
 	QString line;
-
-	//FIXME : counts environment names as words
+	
 	if ( m_doc && m_doc->hasSelection() )
 	{
 		line = m_doc->selection();
+		kdDebug() << "getStat : line : " << line << endl;
 		count(line, m_arStatistics);
 	}
 	else if (m_doc)
@@ -387,7 +585,7 @@ const long* TeXInfo::getStatistics()
 	}
 	return m_arStatistics;
 }
-
+ 
 BracketResult TeXInfo::matchBracket(uint &l, uint &pos)
 {
 	BracketResult result;
@@ -398,7 +596,7 @@ BracketResult TeXInfo::matchBracket(uint &l, uint &pos)
 		int p = 0;
 		while ( l < m_doc->numLines() )
 		{
-			if ( (p = m_doc->textLine(l).find('{', pos)) != -1 )
+			if ( (p = getTextline(l).find('{', pos)) != -1 )
 			{
 				pos = p;
 				break;
@@ -412,7 +610,11 @@ BracketResult TeXInfo::matchBracket(uint &l, uint &pos)
 	}
 
 	if ( m_doc->textLine(l)[pos] == '{' )
+	{
+		result.line = l;
+		result.col = pos;
 		result.value  = Info::matchBracket('{', l, pos);
+	}
 
 	return result;
 }
@@ -420,7 +622,7 @@ BracketResult TeXInfo::matchBracket(uint &l, uint &pos)
 //FIXME refactor, clean this mess up
 void TeXInfo::updateStruct()
 {
-	kdDebug() << "==void TeXInfo::updateStruct()=========." << endl;
+	kdDebug() << "==void TeXInfo::updateStruct: (" << url() << ")=========" << endl;
 
 	if ( getDoc() == 0L ) return;
 
@@ -428,11 +630,11 @@ void TeXInfo::updateStruct()
 
 	QMapConstIterator<QString,KileStructData> it;
 	static QRegExp::QRegExp reCommand("(\\\\[a-zA-Z]+)\\s*\\*?\\s*(\\{|\\[)");
-	static QRegExp::QRegExp reComments("([^\\\\]%|^%).*$");
 	static QRegExp::QRegExp reRoot("\\\\documentclass|\\\\documentstyle");
 	static QRegExp::QRegExp reBD("\\\\begin\\s*\\{\\s*document\\s*\\}");
 	static QRegExp::QRegExp reReNewCommand("\\\\renewcommand.*$");
 	static QRegExp::QRegExp reNumOfParams("\\s*\\[([1-9]+)\\]");
+	static QRegExp::QRegExp reNumOfOptParams("\\s*\\[([1-9]+)\\]\\s*\\[([^\\{]*)\\]"); // the quantifier * isn't used by mistake, because also emtpy optional brackets are correct.
 
 	int teller=0, tagStart, bd = 0;
 	uint tagEnd, tagLine = 0, tagCol = 0;
@@ -440,13 +642,11 @@ void TeXInfo::updateStruct()
 	QString m, s, shorthand;
 	bool foundBD = false; // found \begin { document }
 	bool fire = true; //whether or not we should emit a foundItem signal
+	bool fireSuspended; // found an item, but it should not be fired (this time)
+	bool depsHasChanged = false;
 
 	for(uint i = 0; i < m_doc->numLines(); ++i)
 	{
-		tagStart=tagEnd=0;
-		s=m_doc->textLine(i);
-		fire = true;
-
 		if (teller > 100)
 		{
 			teller=0;
@@ -455,11 +655,11 @@ void TeXInfo::updateStruct()
 		else
 			++teller;
 
-		//remove escaped \ characters
-		s.replace("\\\\", "  ");
-
-		//remove comments
-		s.replace(reComments, "");
+		tagStart=tagEnd=0;
+		fire = true;
+		s = getTextline(i);
+		if ( s.isEmpty() )
+			continue;
 
 		//ignore renewcommands
 		s.replace(reReNewCommand, "");
@@ -477,7 +677,7 @@ void TeXInfo::updateStruct()
 
 			if ((!foundBD) && (s.find(reRoot, tagEnd) != -1))
 			{
-				kdDebug() << "\tsetting m_bIsRoot to TRUE" << endl;
+				kdDebug() << "\tsetting m_bIsRoot to true" << endl;
 				tagEnd += reRoot.cap(0).length();
 				m_bIsRoot = true;
 			}
@@ -502,15 +702,67 @@ void TeXInfo::updateStruct()
 					shorthand = result.option.stripWhiteSpace();
 					if ( i >= tagLine ) //matching brackets spanned multiple lines
 						s = m_doc->textLine(i);
-					kdDebug() << "\tgrabbed : " << m << endl;
+					if ( result.line>0 || result.col>0 )
+					{
+						tagLine = result.line + 1;
+						tagCol = result.col + 1;
+					}
+					//kdDebug() << "\tgrabbed: " << reCommand.cap(1) << "[" << shorthand << "]{" << m << "}" << endl;
 				}
 
 				//title (or label) found, add the element to the listview
-				if (!m.isNull())
+				if ( !m.isNull() )
 				{
+					// no problems so far ...
+					fireSuspended = false;
+					
+					// remove trailing ./ 
+					if ( (*it).type & (KileStruct::Input | KileStruct::Graphics) )
+					{
+						if ( m.left(2) == "./" )
+							m = m.mid(2,m.length()-2);
+					}
+					// update parameter for environments, because only
+					// floating environments are passed
+					if ( (*it).type == KileStruct::BeginEnv )
+					{
+						if ( m=="figure" || m=="table")
+							it = m_dictStructLevel.find("\\begin{"+m+"}");
+						else
+							fireSuspended = true;          // only floats, no other environments
+					}
+					
+					// tell structure view that a floating environment must be closed
+					else if ( (*it).type == KileStruct::EndEnv )
+					{
+						if ( m=="figure" || m=="table")
+							it = m_dictStructLevel.find("\\end{float}");
+						else
+							fireSuspended = true;          // only floats, no other environments
+					}
+					
+					// sectioning commands
+					else if ( (*it).type == KileStruct::Sect ) 
+					{
+						if ( ! shorthand.isNull() )
+							m = shorthand;
+					}
 
-					//update the dependencies
-					if ((*it).type == KileStruct::Input)
+					// update the label list
+					else if ( (*it).type == KileStruct::Label )
+					{
+						m_labels.append(m);
+					}
+
+					// update the references list
+					else if ( (*it).type == KileStruct::Reference )
+					{
+						// m_references.append(m);
+						//fireSuspended = true;          // don't emit references
+					}
+
+					// update the dependencies
+					else if ((*it).type == KileStruct::Input)
 					{
 						QString dep = m;
 						if (dep.right(4) != ".tex")
@@ -518,35 +770,51 @@ void TeXInfo::updateStruct()
 						m_deps.append(dep);
 					}
 
-					//update the referenced Bib files
-					if((*it).type == KileStruct::Bibliography)
+					// update the referenced Bib files
+					else  if( (*it).type == KileStruct::Bibliography )
 					{
-						kdDebug() << "\tappending Bibiliograph file " << m << endl;
+						kdDebug() << "===TeXInfo::updateStruct()===appending Bibiliograph file(s) " << m << endl;
+
+						if( m_prevbib != m && !m_prevbib.isEmpty() && !m.isEmpty() )
+							depsHasChanged=true;
+						kdDebug() << "depsHasChanged = " << depsHasChanged << endl;
+						m_prevbib=m;
+
 						QStringList bibs = QStringList::split(",", m);
+						QString biblio;
+						
 						uint cumlen = 0;
+						uint nextbib = 0; // length to add to jump to the next bibliography
 						for (uint b = 0; b < bibs.count(); ++b)
 						{
-							m_bibliography.append(bibs[b]);
-							m_deps.append(bibs[b] + ".bib");
-							emit(foundItem(bibs[b], tagLine, tagCol + cumlen, (*it).type, (*it).level, (*it).pix, (*it).folder));
-							cumlen += bibs[b].length() + 1;
+							nextbib = 0;
+							biblio=bibs[b];
+							m_bibliography.append(biblio);
+							if ( biblio.left(2) == "./" )
+							{	nextbib += 2; 
+								biblio = biblio.mid(2,biblio.length()-2);
+							}
+							if ( biblio.right(4) == ".bib" )
+							{
+								biblio =biblio.left(biblio.length()-4); 
+								nextbib +=4;
+							}
+							m_deps.append(biblio + ".bib");
+							emit(foundItem(biblio, tagLine, tagCol + cumlen, (*it).type, (*it).level, (*it).pix, (*it).folder));
+							cumlen += biblio.length() + 1 + nextbib;
 						}
 						fire = false;
 					}
 
-					//update the label list
-					if ((*it).type == KileStruct::Label)
-						m_labels.append(m);
-
-					//update the bibitem list
-					if ((*it).type == KileStruct::BibItem)
+					// update the bibitem list
+					else if ( (*it).type == KileStruct::BibItem )
 					{
-						kdDebug() << "\tappending bibitem " << m << endl;
+						//kdDebug() << "\tappending bibitem " << m << endl;
 						m_bibItems.append(m);
 					}
 
-					//update the package list
-					if ((*it).type == KileStruct::Package)
+					// update the package list
+					else if ( (*it).type == KileStruct::Package )
 					{
 						QStringList pckgs = QStringList::split(",", m);
 						uint cumlen = 0;
@@ -555,42 +823,62 @@ void TeXInfo::updateStruct()
 							QString package = pckgs[p].stripWhiteSpace();
 							if ( ! package.isEmpty() ) {
 								m_packages.append(package);
-								emit(foundItem(package, tagLine, tagCol + cumlen, (*it).type, (*it).level, (*it).pix, (*it).folder));
+								// hidden, so emit is useless
+								// emit(foundItem(package, tagLine, tagCol + cumlen, (*it).type, (*it).level, (*it).pix, (*it).folder));
 								cumlen += package.length() + 1;
 							}
 						}
 						fire = false;
 					}
 
-					//newcommand found, add it to the newCommands list
-					if ( (*it).type == KileStruct::NewCommand )
+					// newcommand found, add it to the newCommands list
+					else if ( (*it).type == KileStruct::NewCommand )
 					{
 						//find how many parameters this command takes
 						if ( s.find(reNumOfParams, tagEnd + 1) != -1 )
 						{
 							bool ok;
 							int noo = reNumOfParams.cap(1).toInt(&ok);
+							QString cmdWithOptArgs = QString::null;
 							if ( ok )
 							{
+								if(s.find(reNumOfOptParams, tagEnd + 1) != -1)
+								{
+								kdDebug() << "Opt param is " << reNumOfOptParams.cap(2) << "%EOL" << endl;
+								noo--; // if we have an opt argument, we have one mandatory argument less, and noo=0 can't occure because then latex complains (and we don't macht them with reNumOfParams either)
+								cmdWithOptArgs = m + "[" + reNumOfOptParams.cap(2) + "]";
+								}
+									
 								for ( int noo_index = 0; noo_index < noo; ++noo_index)
+								{
 									m +=  "{" + s_bullet + "}";
+									if(!cmdWithOptArgs.isNull())
+										cmdWithOptArgs += "{" + s_bullet + "}";
+								}
+								
 							}
+						if(!cmdWithOptArgs.isNull())
+							m_newCommands.append(cmdWithOptArgs);  // if we have opt args we add two new commands, one with and one without opt args.
 						}
 						m_newCommands.append(m);
-						//ignore rest of line
-						continue;
+						//FIXME  set tagEnd to the end of the command definition
+						break;
 					}
 
-					if ( ((*it).type == KileStruct::Sect) && (!shorthand.isNull()) )
-						m = shorthand;
+					// and some other commands, which don't need special actions: 
+					// \caption, ...
 
-					kdDebug() << "emitting foundItem " << m << endl;
-					if (fire) emit(foundItem(m, tagLine, tagCol, (*it).type, (*it).level, (*it).pix, (*it).folder));
+					//kdDebug() << "\t\temitting: " << m << endl;
+					if ( fire && !fireSuspended ) 
+						emit( foundItem(m, tagLine, tagCol, (*it).type, (*it).level, (*it).pix, (*it).folder) );
 				} //if m
 			} // if tagStart
 		} // while tagStart
 	} //for
 
+	if(depsHasChanged) 
+		emit(depChanged()); // does the order of the signals matter ?
+				// TODO do the same kind of smart update for \input and \includegraphics
 	emit(doneUpdating());
 	emit(isrootChanged(isLaTeXRoot()));
 }
@@ -668,45 +956,4 @@ void BibInfo::updateStruct()
 
 }
 
-/*
- *
- *	KileDocInfoDlg : a dialog that displays information known about the current document
- *
- *
- */
-
-KileDocInfoDlg::KileDocInfoDlg(KileDocument::Info *docinfo, QWidget* parent,  const char* name, const QString &caption)
-	: KDialogBase(parent,name,true,caption,KDialogBase::Ok, KDialogBase::Ok, true)
-{
-	QWidget *page = new QWidget( this );
-	setMainWidget(page);
-	QGridLayout *layout = new QGridLayout( page, 8, 3,5,5,"");
-
-	const long *list = docinfo->getStatistics();
-
-	layout->addWidget(new QLabel(i18n("Characters in words:"),page), 0,0);
-	layout->addWidget(new QLabel(QString::number(list[0]), page), 0,2);
-
-	layout->addWidget(new QLabel(i18n("Characters in LaTeX commands:"),page), 1,0);
-	layout->addWidget(new QLabel(QString::number(list[1]), page), 1,2);
-
-	layout->addWidget(new QLabel(i18n("Whitespace/Delimiters/Punctuation marks:"),page), 2,0);
-	layout->addWidget(new QLabel(QString::number(list[2]), page), 2,2);
-
-	layout->addWidget(new QLabel(i18n("Total:"),page), 3,1, Qt::AlignRight);
-	layout->addWidget(new QLabel(QString::number(list[0]+list[1]+list[2]), page), 3,2);
-
-	layout->addWidget(new QLabel(i18n("Words:"),page), 4,0);
-	layout->addWidget(new QLabel(QString::number(list[3]), page), 4,2);
-
-	layout->addWidget(new QLabel(i18n("LaTeX commands:"),page), 5,0);
-	layout->addWidget(new QLabel(QString::number(list[4]), page), 5,2);
-
-	layout->addWidget(new QLabel(i18n("Total:"),page), 6,1, Qt::AlignRight);
-	layout->addWidget(new QLabel(QString::number(list[3]+list[4]), page), 6,2);
-
-	if (docinfo->getDoc() && docinfo->getDoc()->hasSelection())
-		layout->addWidget(new QLabel(i18n("WARNING: These are the statistics for the selected text only."),page), 7,0, Qt::AlignRight);
-}
-
 #include "kiledocumentinfo.moc"
