@@ -1,7 +1,7 @@
 /***************************************************************************
-    date                 : Mar 05 2005
-    version              : 0.11
-    copyright            : (C) 2004-2005 by Holger Danielsson
+    date                 : Feb 05 2006
+    version              : 0.27
+    copyright            : (C) 2004-2006 by Holger Danielsson
     email                : holger.danielsson@t-online.de
  ***************************************************************************/
 
@@ -15,6 +15,7 @@
  ***************************************************************************/
 
 #include <qfileinfo.h>
+#include <qvaluestack.h>
 
 #include <kate/view.h>
 #include <kate/document.h>
@@ -23,6 +24,7 @@
 #include <klocale.h>
 #include <kinputdialog.h>
 
+#include "kilekonsolewidget.h"
 #include "kileinfo.h"
 #include "kileviewmanager.h"
 #include "kileconfig.h"
@@ -36,48 +38,22 @@ namespace KileDocument
 EditorExtension::EditorExtension(KileInfo *info) : m_ki(info)
 {
 	m_complete = new KileDocument::CodeCompletion(m_ki);
+	m_latexCommands = m_ki->latexCommands();
 
 	// init regexp
-	m_reg.setPattern("\\\\(begin|end)\\s*\\{\\s*([A-Za-z]+\\*?)\\s*\\}");
+	m_reg.setPattern("(\\\\(begin|end)\\s*\\{([A-Za-z]+\\*?)\\})|(\\\\\\[|\\\\\\])");
+	//                1    2                 3                   4         
+	m_regexpEnter.setPattern("^(.*)((\\\\begin\\s*\\{([^\\{\\}]*)\\})|(\\\\\\[))");
+	//                         1   23                 4               5 
 
-	// init environments
-	QStringList listenv;
-	listenv << "description"  << "enumerate"  << "itemize"
-	        << "description*" << "enumerate*" << "itemize*"         // mdwtab
-	        << "Bdescription" << "Benumerate" << "Bitemize"         // fancybox
-	        << "labeling"                                           // koma-script
-	        ; 
-	/*
-	QStringList mathenv;
-	mathenv << "align"  << "alignat" << << "alignat*" 
-	        << "aligned" << "alignedat"
-	        << "bmatrix"
-	        << "cases"
-	        << "eqnarray" << "eqnarray*"
-	        << "flalign" << "flalign*"
-	        << "gather" << "gather*" << "gathered"
-	        << "matrix" << "multline" << "multline*"
-	        << "pmatrix"
-	        << "split"
-	        << "vmatrix" << "Vmatrix"
-	        << "xalignat" << "xalignat*" << "xxalignat"<< "xxalignat*"
-	        ;
-	 
-	QStringList tabularenv;
-	tabularenv << "array" << "tabular" << "tabular*" << "tabbing"   
-	           << "longtable"                                       // longtable
-	           << "supertabular" << "supertabular*"                 // supertab
-	           << "mpsupertabular*" << "mpsupertabular*"            // supertab
-	           << "tabularx"                                        // tabularx             
-	           << "xtabular" << "mpxtabular"                        // xtab
-	           << "blockarray" << "block"                           // blkarray
-	           ;
-	*/
-	
-	// set standard environments
-	setEnvironment(listenv,m_dictListEnv);
-	// setEnvironment(mathenv,m_dictMathEnv);
-	// setEnvironment(tabularenv,m_dictTabularEnv);
+	// init double quotes
+	m_quoteList 
+		<< "English quotes:   ``   ''"
+		<< "French quotes:   \"<   \">"
+		<< "German quotes:   \"`   \"'"
+		<< "French quotes (long):   \\flqq   \\frqq"
+		<< "German quotes (long):   \\glqq   \\grqq"
+		;
 
 	readConfig();
 }
@@ -91,10 +67,25 @@ EditorExtension::~EditorExtension()
 
 void EditorExtension::readConfig(void)
 {
-	// config section
-	setEnvironment(KileConfig::envList(), m_dictListEnv);
-	setEnvironment(KileConfig::envMath(), m_dictMathEnv);
-	setEnvironment(KileConfig::envTabular(), m_dictTabularEnv);
+	// init insertion of double quotes
+	initDoubleQuotes();
+
+	// calculate indent for autoindent of environments
+	m_envAutoIndent = QString::null;
+	if ( KileConfig::envIndentation() )
+	{
+		if ( KileConfig::envIndentSpaces() )
+		{
+			int num = KileConfig::envIndentNumSpaces();
+			if ( num<1 || num>9 )
+				num = 1;
+			m_envAutoIndent.fill(' ',num);
+		}
+		else
+		{
+			m_envAutoIndent = "\t";
+		}
+	}
 }
 
 void EditorExtension::insertTag(const KileAction::TagData& data, Kate::View *view)
@@ -117,6 +108,12 @@ void EditorExtension::insertTag(const KileAction::TagData& data, Kate::View *vie
 	int para_end=0;
 	int index_cursor=index;
 	int para_cursor=index;
+	// offset for autoindentation of environments
+	int dxIndentEnv = 0;
+
+	// environment tag
+	bool envtag = data.tagBegin.contains("%E") || data.tagEnd.contains("%E");
+	QString whitespace = getWhiteSpace( doc->textLine(para).left(index) );
 
 	//if there is a selection act as if cursor is at the beginning of selection
 	if (wrap)
@@ -127,24 +124,57 @@ void EditorExtension::insertTag(const KileAction::TagData& data, Kate::View *vie
 	}
 
 	QString ins = data.tagBegin;
+	QString tagEnd = data.tagEnd;
 
 	//start an atomic editing sequence
 	KTextEditor::EditInterfaceExt *editInterfaceExt = KTextEditor::editInterfaceExt( doc );
 	if ( editInterfaceExt ) editInterfaceExt->editBegin();
 
 	//cut the selected text
+	QString trailing;
 	if (wrap)
 	{
-		ins += doc->selection();
+		QString sel = doc->selection();
 		doc->removeSelectedText();
+		
+		// no autoindentation of environments, when text is selected
+		if ( envtag )
+		{
+			ins.remove("%E");
+			tagEnd.remove("%E\n");
+		}
+
+		// strip one of two consecutive line ends
+		int len = sel.length();
+		if ( tagEnd.at(0)=='\n' && len>0 && sel.find('\n',-1)==len-1 )
+			sel.truncate( len-1 );
+			
+		// now add the selection
+		ins += sel;
+		
+		// place the cursor behind this tag, if there is no other wish
+		if ( !before && !after )
+		{
+			trailing = "%C";
+			after = true;
+		}
+	}
+	else if ( envtag  )
+	{
+		ins.replace("%E",whitespace+m_envAutoIndent);
+		tagEnd.replace("%E",whitespace+m_envAutoIndent);
+		if ( data.dy > 0 )
+			dxIndentEnv = whitespace.length() + m_envAutoIndent.length();
 	}
 
-	ins += data.tagEnd;
+	tagEnd.replace("\\end{",whitespace+"\\end{");
+	ins += tagEnd + trailing;
 
 	//do some replacements
 	QFileInfo fi( doc->url().path());
 	ins.replace("%S", fi.baseName(true));
-
+	ins.replace("%B", s_bullet);
+	
 	//insert first part of tag at cursor position
 	doc->insertText(para,index,ins);
 
@@ -174,7 +204,7 @@ void EditorExtension::insertTag(const KileAction::TagData& data, Kate::View *vie
 			py = para;
 			px = index;
 		}
-		para_cursor = py+data.dy; index_cursor = px+data.dx;
+		para_cursor = py+data.dy; index_cursor = px+data.dx+dxIndentEnv;
 	}
 
 	//end the atomic editing sequence
@@ -185,28 +215,6 @@ void EditorExtension::insertTag(const KileAction::TagData& data, Kate::View *vie
 
 	doc->clearSelection();
 }
-//////////////////// list/math/tabular environments ////////////////////
-
-void EditorExtension::setEnvironment(const QStringList &list, QMap<QString,bool> &map)
-{
-	for (uint i=0; i<list.count(); ++i)
-		map[list[i]] = true;
-}
-
-bool EditorExtension::isListEnvironment(const QString &name)
-{
-	return m_dictListEnv.contains(name);
-}
-
-bool EditorExtension::isMathEnvironment(const QString &name)
-{
-	return m_dictMathEnv.contains(name);
-}
-
-bool EditorExtension::isTabEnvironment(const QString &name)
-{
-	return m_dictTabularEnv.contains(name);
-}
 
 //////////////////// goto environment tag (begin or end) ////////////////////
 
@@ -288,7 +296,10 @@ void EditorExtension::closeEnvironment(Kate::View *view)
 	
 	if ( findOpenedEnvironment(row,col,name,view) )
 	{
-		view->getDoc()->insertText( row,col, "\\end{"+name+"}" );
+		if ( name == "\\[" )
+			view->getDoc()->insertText( row,col, "\\]" );
+		else
+			view->getDoc()->insertText( row,col, "\\end{"+name+"}" );
 // 		view->setCursorPositionReal(row+1,0);
 	}
 }
@@ -308,27 +319,23 @@ void EditorExtension::insertIntelligentNewline(Kate::View *view)
 
 	if ( findOpenedEnvironment(row,col,name,view) )
 	{
-		if ( isListEnvironment(name) )
+		//kdDebug() << "   name==" << name << " " << m_latexCommands->isListEnv(name) << endl;
+		if ( m_latexCommands->isListEnv(name) )
 		{
 			view->keyReturn();
 			view->insertText("\\item " );
 			return;
-			//view->getDoc()->insertText( row,col,"\n\\item " );
-			//view->setCursorPositionReal(row + 1,6);
 		}
-// 		else if ( isTabEnvironment(name) || isMathEnvironment(name) )
-// 		{
-// 			view->getDoc()->insertText( row,col,"\\\\\n" );
-// 			view->setCursorPositionReal(row + 1,0);
-// 		}
-		//else
-		//	view->getDoc()->insertText( row,col,"\\\\\n" );
+		else if ( m_latexCommands->isTabularEnv(name) || m_latexCommands->isMathEnv(name) )
+		{
+			view->insertText(" \\\\");
+		}
 	}
-	//else
-	//	view->getDoc()->insertText( row,col,"\\\\\n" );
-	view->insertText("\\\\");
+	
+	// - found no opened environment
+	// - unknown environment
+	// - finish tabular or math environment
 	view->keyReturn();
-
 }
 
 bool EditorExtension::findOpenedEnvironment(uint &row,uint &col, QString &envname, Kate::View *view)
@@ -555,7 +562,7 @@ bool EditorExtension::findBeginEnvironment(Kate::Document *doc, uint row, uint c
 
 //////////////////// search for \end{env}  ////////////////////
 
-// Find the last \begin{env} tag. If the current cursor is over
+// Find the last \end{env} tag. If the current cursor is over
 //  - \end{env} tag: we will stop immediately
 //  - \begin{env} tag: we will start behind this tag
 
@@ -583,18 +590,20 @@ bool EditorExtension::findEndEnvironment(Kate::Document *doc, uint row, uint col
 bool EditorExtension::findEnvironmentTag(Kate::Document *doc, uint row, uint col,
                                   EnvData &env,bool backwards)
 {
+	//kdDebug() << "findEnvTag " << backwards << endl;
 	KTextEditor::SearchInterface *iface;
 	iface = dynamic_cast<KTextEditor::SearchInterface *>(doc);
-	//QRegExp reg("\\\\(begin|end)\\s*\\{\\s*([A-Za-z]+\\*?)\\s*\\}");
 	
 	uint envcount = 0;
-	QString wrong_env = ( backwards ) ? "end" : "begin";
+	
+	EnvTag wrong_env = ( backwards ) ? EnvEnd : EnvBegin;
 	while ( iface->searchText(row,col,m_reg,&env.row,&env.col,&env.len,backwards) )
 	{
 		//   kdDebug() << "   iface " << env.row << "/" << env.col << endl;
 		if ( isValidBackslash(doc,env.row,env.col) )
 		{
-			if ( m_reg.cap(1) == wrong_env )
+			EnvTag found_env = ( m_reg.cap(2)=="begin" || m_reg.cap(4)=="\\[" ) ? EnvBegin : EnvEnd; 
+			if ( found_env == wrong_env )
 			{
 				++envcount;
 			}
@@ -604,7 +613,15 @@ bool EditorExtension::findEnvironmentTag(Kate::Document *doc, uint row, uint col
 					--envcount;
 				else
 				{
-					env.name = m_reg.cap(2);
+					if ( found_env == EnvBegin ) 
+					{
+						env.name = ( m_reg.cap(2)=="begin" ) ? m_reg.cap(3) : "\\[";
+					}
+					else
+					{
+						env.name = ( m_reg.cap(2)=="end" ) ? m_reg.cap(3) : "\\]";
+					}	
+					env.tag = found_env;
 					return true;
 				}
 			}
@@ -646,12 +663,12 @@ bool EditorExtension::isEnvironmentPosition(Kate::Document *doc, uint row, uint
 	bool left = false;
 	bool right = false;
 	
-	KTextEditor::SearchInterface *iface;
-	iface = dynamic_cast<KTextEditor::SearchInterface *>(doc);
+	//KTextEditor::SearchInterface *iface;
+	//iface = dynamic_cast<KTextEditor::SearchInterface *>(doc);
 	
 	// check if there is a match in this line from the current position to the left
 	int startcol = ( textline[col] == '\\' ) ? col - 1 : col;
-	if ( startcol > 6 )
+	if ( startcol >= 1 )
 	{
 		int pos = textline.findRev(m_reg,startcol);
 		env.len = m_reg.matchedLength();
@@ -660,8 +677,17 @@ bool EditorExtension::isEnvironmentPosition(Kate::Document *doc, uint row, uint
 		{
 			env.row = row;
 			env.col = pos;
-			env.tag = ( textline.at(pos+1) == 'b' ) ? EnvBegin : EnvEnd;
-			env.name = m_reg.cap(2);
+			QChar ch = textline.at(pos+1);
+			if ( ch=='b' || ch=='e' ) 
+			{
+				env.tag = ( ch == 'b' ) ? EnvBegin : EnvEnd;
+				env.name = m_reg.cap(3);
+			} 
+			else
+			{
+				env.tag = ( ch == '[' ) ? EnvBegin : EnvEnd;
+				env.name = m_reg.cap(4);
+			}
 			env.cpos =  ( col < (uint)pos+env.len ) ? EnvInside : EnvRight;
 			// we have already found a tag, if the cursor is inside, but not behind this tag
 			if ( env.cpos == EnvInside )
@@ -671,19 +697,30 @@ bool EditorExtension::isEnvironmentPosition(Kate::Document *doc, uint row, uint
 		}
 	}
 	
+	//kdDebug() << "search right " << endl;
 	// check if there is a match in this line from the current position to the right
 	if ( textline[col]=='\\' && col==(uint)textline.find(m_reg,col) )
 	{
 		envright.row = row;
 		envright.col = col;
 		envright.len = m_reg.matchedLength();
-		envright.tag = ( textline.at(col+1) == 'b' ) ? EnvBegin : EnvEnd;
-		envright.name = m_reg.cap(2);
+		QChar ch = textline.at(col+1);
+		if ( ch=='b' || ch=='e' )        // found "\begin" or "\end"
+		{
+			envright.tag = ( ch == 'b' ) ? EnvBegin : EnvEnd;
+			envright.name = m_reg.cap(3);
+		} 
+		else                             // found "\[" or "\\]"
+		{
+			envright.tag = ( ch == '[' ) ? EnvBegin : EnvEnd;
+			envright.name = m_reg.cap(4);
+		}
 		envright.cpos = EnvLeft;
 		right = true;
 		//kdDebug() << "   is - found right:  pos=" <<col << " " << envright.name << " " << QString(textline.at(col+1)) << endl;
 	}
 	
+	//kdDebug() << "found left/right: " << left << "/" << right << endl;
 	// did we find a tag?
 	if ( ! (left || right) )
 		return false;
@@ -842,7 +879,7 @@ bool EditorExtension::decreaseCursorPosition(Kate::Document *doc, uint &row, uin
 	else if ( row > 0 )
 	{
 		--row;
-		col = doc->lineLength(row) - 1;
+		col = doc->lineLength(row);
 	}
 	else
 		ok = false;
@@ -1072,7 +1109,7 @@ bool EditorExtension::isBracketPosition(Kate::Document *doc, uint row, uint col,
 	QChar right = textline[col];
 	QChar left  = ( col > 0 ) ? textline[col-1] : QChar(' ');
 	
-	kdDebug() << QString("isBracketPosition: (%1,%2) left %3 right %4").arg(row).arg(col).arg(left).arg(right) << endl;
+	//kdDebug() << QString("isBracketPosition: (%1,%2) left %3 right %4").arg(row).arg(col).arg(left).arg(right) << endl;
 	if ( m_overwritemode )
 	{
 		if ( right == '{' )
@@ -1154,7 +1191,7 @@ bool EditorExtension::findOpenBracketTag(Kate::Document *doc, uint row, uint col
 		int start = ( line == (int)row ) ? col : textline.length()-1;
 		for ( int i=start; i>=0; --i )
 		{
-			kdDebug() << "findOpenBracketTag: (" << line << "," << i << ") = " << textline[i].latin1() << endl;
+			//kdDebug() << "findOpenBracketTag: (" << line << "," << i << ") = " << textline[i].latin1() << endl;
 			if ( textline[i] == '{' )
 			{
 				if ( brackets > 0 )
@@ -1174,7 +1211,7 @@ bool EditorExtension::findOpenBracketTag(Kate::Document *doc, uint row, uint col
 		}
 	}
 	
-	kdDebug() << "nothting found" << endl;
+	//kdDebug() << "nothting found" << endl;
 	return false;
 }
 
@@ -1316,130 +1353,6 @@ bool EditorExtension::getCurrentWord(Kate::Document *doc, uint row, uint col, Ed
 }
 
 
-//////////////////// move/unmove selections ////////////////////
-
-void EditorExtension::commentSelection(bool insert, Kate::View *view)
-{
-	view = determineView(view);
-	if ( !view ) return;
-
-	moveSelection("%",insert,view);
-}
-
-void EditorExtension::spaceSelection(bool insert, Kate::View *view)
-{
-	view = determineView(view);
-	if ( !view ) return;
-
-	moveSelection(" ",insert,view);
-}
-
-void EditorExtension::tabSelection(bool insert, Kate::View *view)
-{
-	view = determineView(view);
-	if ( !view ) return;
-
-	moveSelection("\t",insert,view);
-}
-
-void EditorExtension::stringSelection(bool insert, Kate::View *view)
-{
-	view = determineView(view);
-	if ( !view ) return;
-	bool ok;
-	QString text = KInputDialog::getText(i18n("Insert Text"), i18n("Please enter the text to insert:"),"", &ok, view);
-	if ( ok && (!text.isEmpty()))
-	{
-		moveSelection(text, insert, view);
-	}
-}
-
-void EditorExtension::moveSelection(const QString &prefix,bool insertmode, Kate::View *view)
-{
-	view = determineView(view);
-	if ( !view ) return;
-	
-	// get current position
-	uint row,col,row1,col1,row2,col2;
-	Kate::Document *doc = view->getDoc();
-	view->cursorPositionReal(&row,&col);
-	bool blockselectionmode = doc->blockSelectionMode();
-	
-	if ( doc->hasSelection() )
-	{             // selection: all lines in the selected range
-		row1 = doc->selStartLine();
-		col1 = doc->selStartCol();
-		row2 = doc->selEndLine();
-		col2 = doc->selEndCol();
-		if ( !blockselectionmode )
-		{
-		if ( col1 >= doc->textLine(row1).length() )
-			(void) increaseCursorPosition(doc,row1,col1);
-		if ( col2 == 0 )
-			(void) decreaseCursorPosition(doc,row2,col2);
-		}
-	}
-	else
-	{                                // no selection: only one line
-		row1 = row2 = row;
-		col1 = col2 = col;
-	}
-	//    kdDebug() << "   selection:  " << row1 << "/" << col1 << " " << row2 << "/" << col2 << endl;
-	
-	// we always start at column 0, if not in blockselection mode
-	uint startcol = ( blockselectionmode ) ? col1 : 0;
-	uint prefixlen = prefix.length();
-	
-	int changecol  = 0;
-	int changecol1 = 0;
-	int changecol2 = 0;
-	int change = ( insertmode ) ? prefixlen : -prefixlen;
-	for (uint line=row1; line<=row2; ++line)
-	{
-		bool action = true;
-		if ( insertmode )
-		doc->insertText(line,startcol,prefix);
-		else if ( doc->textLine(line).mid(startcol,prefixlen) == prefix )
-		doc->removeText(line,startcol,line,startcol+prefixlen);
-		else
-		action = false;
-	
-		// calculate changes of selection range and cursor position
-		if ( action )
-		{
-		if ( line == row )
-			changecol = change;
-		if ( line == row1 )
-			changecol1 = change;
-		if ( line == row2 )
-			changecol2 = change;
-		}
-	}
-	
-	// also move the selection and the cursor position
-	if ( doc->hasSelection() )
-	{
-		if ( blockselectionmode )
-		doc->setSelection(row1,col1,row2,col2+change);
-		else
-		{
-		if ( col1 > 0 )
-			col1 += changecol1;
-		if ( row > row2 )
-		{
-			++row2;
-			col2 = 0;
-		}
-		else
-			col2 += changecol2;
-		doc->setSelection(row1,col1,row2,col2);
-		}
-	}
-	
-	// finally update cursor position
-	view->setCursorPositionReal(row,col+changecol);
-}
-
 //////////////////// paragraph ////////////////////
 
 void EditorExtension::selectParagraph(Kate::View *view)
@@ -1596,6 +1509,233 @@ void EditorExtension::completeAbbreviation()
 	complete()->editComplete(m_ki->viewManager()->currentView(), KileDocument::CodeCompletion::cmAbbreviation);
 }
 
+//////////////////// double quotes ////////////////////
+
+void EditorExtension::initDoubleQuotes()
+{
+	m_dblQuotes = KileConfig::insertDoubleQuotes();
+	
+	int index = KileConfig::doubleQuotes();
+	if ( index<0 && index>=(int)m_quoteList.count() )
+		index = 0;
+	
+	QStringList quotes = QStringList::split(QRegExp("\\s{2,}"), m_quoteList[index] ); 
+	m_leftDblQuote=  quotes[1];
+	m_rightDblQuote = quotes[2];
+	kdDebug() << "new quotes: " << m_dblQuotes << " left=" << m_leftDblQuote << " right=" << m_rightDblQuote<< endl;
+}
+
+bool EditorExtension::insertDoubleQuotes()
+{
+	// don't insert double quotes, if konsole has focus 
+	// return false, because if this is called from an event
+	// handler, because this event has to be passed on
+	if ( m_ki->texKonsole()->hasFocus() )
+		return false;
+
+	// insert double quotes, normal mode or autocompletion mode
+	// always return true for event handler
+	Kate::View *view = determineView(0L);
+	if ( !view ) return true;
+	
+	uint row,col;
+	view->cursorPositionReal(&row,&col);
+	Kate::Document *doc = view->getDoc();
+	
+	// simply insert, if autoinsert mode is not active or the char bevor is \ (typically for \"a useful)
+	if ( !m_dblQuotes || ( col > 0 && doc->text(row,col-1,row,col) == QString("\\") ) ) 
+	{
+		doc->insertText(row,col,"\"");
+		return true;
+	}
+
+	// insert with auto mode
+	KTextEditor::SearchInterface *iface;
+	iface = dynamic_cast<KTextEditor::SearchInterface *>(doc);
+		
+	QString pattern1 = QRegExp::escape(m_leftDblQuote);
+	if ( m_leftDblQuote.at(m_leftDblQuote.length()-1).isLetter() )
+		pattern1 += "(\\b|(\\{\\}))";
+	QString pattern2 = QRegExp::escape(m_rightDblQuote);
+	if ( m_rightDblQuote.at(m_rightDblQuote.length()-1).isLetter() )
+		pattern2 += "(\\b|(\\{\\}))";
+
+	QRegExp reg("(" + pattern1 + ")|(" + pattern2 + ")");
+
+	uint r,c,l;
+	bool openfound = false;
+	if ( iface->searchText(row,col,reg,&r,&c,&l,true) )  
+	{
+		openfound = ( doc->textLine(r).find(m_leftDblQuote,c) == (int)c );
+		//kdDebug() << "pattern=" << reg.pattern() << " " << reg.cap(1) << " r=" << r << " c=" << c << " open=" << openfound<< endl;
+	}
+	
+	QString textline = doc->textLine(row);
+	//kdDebug() << "text=" << textline << " open=" << openfound << endl;
+	if ( openfound ) 
+	{
+		// If we last inserted a language specific doublequote open,  
+		// we have to change it to a normal doublequote. If not we 
+		// insert a language specific doublequote close
+		int startcol = col - m_leftDblQuote.length();
+		//kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
+		if ( startcol>=0 && textline.find(m_leftDblQuote,startcol) == (int)startcol ) 
+		{
+				doc->removeText(row,startcol,row,startcol+m_leftDblQuote.length());
+				doc->insertText(row,startcol,"\"");
+		}
+		else
+		{
+			doc->insertText(row,col,m_rightDblQuote);
+		}
+	}
+	else 
+	{
+		// If we last inserted a language specific doublequote close,  
+		// we have to change it to a normal doublequote. If not we 
+		// insert a language specific doublequote open
+		int startcol = col - m_rightDblQuote.length();
+		//kdDebug() << "startcol=" << startcol << " col=" << col  << endl;
+		if ( startcol>=0 && textline.find(m_rightDblQuote,startcol) == (int)startcol ) 
+		{
+			doc->removeText(row,startcol,row,startcol+m_rightDblQuote.length());
+			doc->insertText(row,startcol,"\"");
+		} 
+		else 
+		{
+			doc->insertText(row,col,m_leftDblQuote);
+		}
+	}
+	return true;
+}
+
+//////////////////// insert tabulator ////////////////////
+	
+void EditorExtension::insertIntelligentTabulator()
+{
+	Kate::View *view = determineView(0L);
+	if ( !view ) return;
+	
+	uint row,col,currentRow,currentCol;
+	QString envname,tab;
+	QString prefix = " ";
+	
+	view->cursorPositionReal(&currentRow,&currentCol);
+	if ( findOpenedEnvironment(row,col,envname,view) ) 
+	{
+		// look if this is an environment with tabs
+		tab = m_latexCommands->getTabulator(envname); 
+		
+		// try to align tabulator with textline above
+		if ( currentRow >= 1 ) 
+		{
+			int tabpos = view->getDoc()->textLine(currentRow-1).find('&',currentCol);
+			if ( tabpos >= 0 ) 
+			{
+				currentCol = tabpos;
+				prefix = QString::null;
+			}
+		}
+	}
+	
+	if ( tab == QString::null ) 
+		tab = "&";
+	tab = prefix + tab + " ";
+	
+	view->getDoc()->insertText(currentRow,currentCol,tab);
+	view->setCursorPositionReal(currentRow,currentCol+tab.length());
+}
+
+//////////////////// autocomplete environment ////////////////////
+
+// should we complete the current environment (call from KileEventFilter)
+
+bool EditorExtension::eventInsertEnvironment(Kate::View *view)
+{
+	// don't complete environment, if we are
+	// still working inside the completion box
+	if ( m_complete->inProgress() )
+		return false;
+
+	int row = view->cursorLine();
+	int col = view->cursorColumnReal();
+	QString line = view->getDoc()->textLine(row).left(col);
+
+	int pos = m_regexpEnter.search(line);
+	if (pos != -1 )
+	{
+		line = m_regexpEnter.cap(1);
+		for (uint i=0; i < line.length(); ++i)
+			if ( ! line[i].isSpace() ) line[i] = ' ';
+		
+		QString envname, endenv;
+		if ( m_regexpEnter.cap(2) == "\\[" ) 
+		{
+			envname = m_regexpEnter.cap(2);
+			endenv = "\\]\n";
+		}
+		else
+		{
+			envname = m_regexpEnter.cap(4);
+			endenv = m_regexpEnter.cap(2).replace("\\begin","\\end")+"\n";
+		}
+		
+		if ( shouldCompleteEnv(envname, view) )
+		{
+			QString item =  m_latexCommands->isListEnv(envname) ? "\\item " : QString::null;
+			view->getDoc()->insertText(row,col, '\n'+line+m_envAutoIndent+item +'\n'+line+endenv);
+			view->setCursorPositionReal(row+1, line.length()+m_envAutoIndent.length()+item.length());
+			return true;
+		}
+	}
+	return false;
+}
+
+bool EditorExtension::shouldCompleteEnv(const QString &env, Kate::View *view)
+{
+	kdDebug() << "===EditorExtension::shouldCompleteEnv(...)===" << endl;
+ 	QString envname = env;
+	envname.replace("*","\\*");
+	QRegExp reTestBegin,reTestEnd;
+	if ( envname == "\\[" )
+	{
+		reTestBegin.setPattern("\\\\\\[");
+		reTestEnd.setPattern("\\\\\\]");
+	}
+	else
+	{
+		reTestBegin.setPattern("\\\\begin\\s*\\{" + envname + "\\}");
+		reTestEnd.setPattern("\\\\end\\s*\\{" + envname + "\\}");
+	}
+	
+	int num = view->getDoc()->numLines();
+	int numBeginsFound = 0;
+	int numEndsFound = 0;
+	uint realLine, realColumn;
+	view->cursorPositionReal(&realLine, &realColumn);
+	for ( int i = realLine; i < num; ++i)
+	{
+		numBeginsFound += view->getDoc()->textLine(i).contains(reTestBegin);
+		numEndsFound += view->getDoc()->textLine(i).contains(reTestEnd);
+		if ( (numBeginsFound == 1) && (numEndsFound == 1) ) return false;        
+		else if ( (numEndsFound == 0) && (numBeginsFound > 1) ) return true;
+		else if ( (numBeginsFound > 2) || (numEndsFound > 1) ) return true; //terminate the search
+	}
+    
+	return true;
+}
+
+QString EditorExtension::getWhiteSpace(const QString &s)
+{
+	QString whitespace = s;
+	for ( uint i=0; i<whitespace.length(); ++i )
+	{
+		if ( ! whitespace[i].isSpace() ) 
+			whitespace[i] = ' ';
+	}
+	return whitespace;
+}
+
 }
 
 #include "kileedit.moc"
