@@ -31,7 +31,7 @@
 const int KPV_ID_OPEN = 0, KPV_ID_SAVE = 1, KPV_ID_CLOSE = 2,
 	KPV_ID_OPTIONS = 3, KPV_ID_ADD = 4, KPV_ID_REMOVE = 5,
 	KPV_ID_BUILDTREE = 6, KPV_ID_ARCHIVE = 7, KPV_ID_ADDFILES = 8,
-	KPV_ID_INCLUDE = 9, KPV_ID_OPENWITH = 10;
+	KPV_ID_INCLUDE = 9, KPV_ID_OPENWITH = 10, KPV_ID_OPENALLFILES = 11;
 
 /*
  * KileProjectViewItem
@@ -105,12 +105,16 @@ void KileProjectView::slotClicked(QListViewItem *item)
 			emit(fileSelected(itm->projectItem()));
 		else if ( itm->type() != KileType::Folder )
 		{
-			//determine mimeType and open file with preferred application
-			KMimeType::Ptr pMime = KMimeType::findByURL(itm->url());
-			if ( pMime->name().startsWith("text/") )
-				emit(fileSelected(itm->url()));
-			else
-				KRun::runURL(itm->url(), pMime->name());
+			// don't open project configuration files (*.kilepr)
+			if ( itm->url().path().right(7) != ".kilepr" )
+			{
+				//determine mimeType and open file with preferred application
+				KMimeType::Ptr pMime = KMimeType::findByURL(itm->url());
+				if ( pMime->name().startsWith("text/") )
+					emit(fileSelected(itm->url()));
+				else
+					KRun::runURL(itm->url(), pMime->name());
+			}
 		}
 	}
 }
@@ -170,6 +174,7 @@ void KileProjectView::slotProject(int id)
 			case KPV_ID_CLOSE : emit(closeProject(item->url())); break;
 			case KPV_ID_ARCHIVE : emit(projectArchive(item->url())); break;
 			case KPV_ID_ADDFILES : emit(addFiles(item->url())); break;
+			case KPV_ID_OPENALLFILES : emit(openAllFiles(item->url())); break;
 			default : break;
 		}
 	}
@@ -190,38 +195,53 @@ void KileProjectView::popup(KListView *, QListViewItem *  item, const QPoint &
 {
 	if (item != 0)
 	{
+		KileProjectViewItem *itm = static_cast<KileProjectViewItem*>(item);
+		if ( itm->type() == KileType::Folder )
+			return;
+		 
 		m_popup->clear();
 		m_popup->disconnect();
 
-		KileProjectViewItem *itm = static_cast<KileProjectViewItem*>(item);
 		bool isKilePrFile = false;
 		if (itm->type() != KileType::Project && itm->projectItem() && itm->projectItem()->project())
 			isKilePrFile = itm->projectItem()->project()->url() == itm->url();
 
+		bool insertsep = false; 
 		if (itm->type() == KileType::ProjectExtra)
 		{
-			KPopupMenu *apps = new KPopupMenu( m_popup);
-			m_offerList = KTrader::self()->query(KMimeType::findByURL(itm->url())->name(), "Type == 'Application'");
-			for (uint i=0; i < m_offerList.count(); ++i)
-				apps->insertItem(SmallIcon(m_offerList[i]->icon()), m_offerList[i]->name(), i+1);
+			if ( ! isKilePrFile )
+			{
+				KPopupMenu *apps = new KPopupMenu( m_popup);
+				m_offerList = KTrader::self()->query(KMimeType::findByURL(itm->url())->name(), "Type == 'Application'");
+				for (uint i=0; i < m_offerList.count(); ++i)
+					apps->insertItem(SmallIcon(m_offerList[i]->icon()), m_offerList[i]->name(), i+1);
 
-			apps->insertSeparator();
-			apps->insertItem(i18n("Other..."), 0);
-			connect(apps, SIGNAL(activated(int)), this, SLOT(slotRun(int)));
-			m_popup->insertItem(SmallIcon("fork"), i18n("&Open With"),apps);
+				apps->insertSeparator();
+				apps->insertItem(i18n("Other..."), 0);
+				connect(apps, SIGNAL(activated(int)), this, SLOT(slotRun(int)));
+				m_popup->insertItem(SmallIcon("fork"), i18n("&Open With"),apps);
+				insertsep = true;
+			}
 		}
 
 		if (itm->type() == KileType::File || itm->type() == KileType::ProjectItem)
 		{
-			m_popup->insertItem(SmallIcon("fileopen"), i18n("&Open"), KPV_ID_OPEN);
-			if (!isKilePrFile) m_popup->insertItem(SmallIcon("filesave"), i18n("&Save"), KPV_ID_SAVE);
-			m_popup->insertSeparator();
+			if ( ! m_ki->isOpen(itm->url()) )
+				m_popup->insertItem(SmallIcon("fileopen"), i18n("&Open"), KPV_ID_OPEN);
+			else
+				m_popup->insertItem(SmallIcon("filesave"), i18n("&Save"), KPV_ID_SAVE);
+			insertsep = true;
 		}
 
 		if (itm->type() == KileType::File)
 		{
-			if (m_nProjects>0) m_popup->insertItem(i18n("&Add to Project"), KPV_ID_ADD);
-			m_popup->insertSeparator();
+			if ( m_nProjects > 0)
+			{
+				if ( insertsep )
+					m_popup->insertSeparator();
+			   m_popup->insertItem(SmallIcon("project_add"),i18n("&Add to Project"), KPV_ID_ADD);
+			   insertsep = true;
+			}
 			connect(m_popup,  SIGNAL(activated(int)), this, SLOT(slotFile(int)));
 		}
 		else if (itm->type() == KileType::ProjectItem || itm->type() == KileType::ProjectExtra)
@@ -229,25 +249,42 @@ void KileProjectView::popup(KListView *, QListViewItem *  item, const QPoint &
 			KileProjectItem *pi  = itm->projectItem();
 			if (pi)
 			{
+				if ( insertsep )
+					m_popup->insertSeparator();
 				m_popup->insertItem(i18n("&Include in Archive"), KPV_ID_INCLUDE);
-				m_popup->insertSeparator();
 				m_popup->setItemChecked(KPV_ID_INCLUDE, pi->archive());
+				insertsep = true;
 			}
-			if (!isKilePrFile) m_popup->insertItem(i18n("&Remove From Project"), KPV_ID_REMOVE);
-			m_popup->insertSeparator();
-			connect(m_popup,  SIGNAL(activated(int)), this, SLOT(slotProjectItem(int)));
+			if ( !isKilePrFile ) 
+			{
+				if ( insertsep )
+					m_popup->insertSeparator();
+				m_popup->insertItem(SmallIcon("project_remove"),i18n("&Remove From Project"), KPV_ID_REMOVE);
+				insertsep = true;
+			}
+   			connect(m_popup,  SIGNAL(activated(int)), this, SLOT(slotProjectItem(int)));
 		}
 		else if (itm->type() == KileType::Project)
 		{
+			if ( insertsep )
+				m_popup->insertSeparator();
 			m_popup->insertItem(i18n("A&dd Files..."), KPV_ID_ADDFILES);
-			m_popup->insertItem(SmallIcon("relation"),i18n("Refresh Project &Tree"), KPV_ID_BUILDTREE);
-   			m_popup->insertItem(SmallIcon("configure"),i18n("Project &Options"), KPV_ID_OPTIONS);
-			m_popup->insertItem(SmallIcon("package"),i18n("&Archive"), KPV_ID_ARCHIVE);
 			m_popup->insertSeparator();
+			m_popup->insertItem(i18n("Open All &Project Files"), KPV_ID_OPENALLFILES);
+			m_popup->insertSeparator();
+			m_popup->insertItem(SmallIcon("relation"),i18n("Refresh Project &Tree"), KPV_ID_BUILDTREE);
+			m_popup->insertItem(SmallIcon("configure"),i18n("Project &Options"), KPV_ID_OPTIONS);
+			m_popup->insertItem(SmallIcon("package"),i18n("&Archive"), KPV_ID_ARCHIVE);
 			connect(m_popup,  SIGNAL(activated(int)), this, SLOT(slotProject(int)));
+			insertsep = true;
 		}
 
-		if ( (itm->type() == KileType::File) || (itm->type() == KileType::ProjectItem) || (itm->type()== KileType::Project)) m_popup->insertItem(SmallIcon("fileclose"), i18n("&Close"), KPV_ID_CLOSE);
+		if ( (itm->type() == KileType::File) || (itm->type() == KileType::ProjectItem) || (itm->type()== KileType::Project))
+		{
+			if ( insertsep )
+				m_popup->insertSeparator();
+			m_popup->insertItem(SmallIcon("fileclose"), i18n("&Close"), KPV_ID_CLOSE);
+		}
 
 		m_popup->exec(point);
 	}
@@ -421,7 +458,6 @@ KileProjectViewItem* KileProjectView::add(KileProjectItem *projitem, KileProject
 	KileProjectViewItem *item, *parent;
 
 	switch (projitem->type()) {
-	case (KileProjectItem::ProjectFile):
 	case (KileProjectItem::Source):
 		item = new KileProjectViewItem(projvi, projitem);
 		item->setType(KileType::ProjectItem);
@@ -431,6 +467,7 @@ KileProjectViewItem* KileProjectView::add(KileProjectItem *projitem, KileProject
 		item = new KileProjectViewItem(parent, projitem);
 		item->setType(KileType::ProjectItem);
 		break;
+	case (KileProjectItem::ProjectFile):
 	default:
 		parent = folder(projitem, projvi);
 		item = new KileProjectViewItem(parent, projitem);
